'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _escope = require('escope');

var _functionType = require('./utils/functionType');

var emptyFn = function emptyFn() {}; // eslint-disable-line no-empty-function

/**
 * A helper for traversing with scope info from escope.
 *
 * Usage:
 *
 *     traverser.traverse(ast, withScope(ast, {
 *       enter(node, parent, scope) {
 *         // do something with node and scope
 *       }
 *     }))
 *
 * @param {Object} ast The AST root node also passed to traverser.
 * @param {Object} cfg Object with enter/leave function as expected by traverser.
 * @return {Object} Object with enter/leave functions to be passed to traverser.
 */
function withScope(ast, _ref) {
  var _ref$enter = _ref.enter;

  var _enter = _ref$enter === undefined ? emptyFn : _ref$enter;

  var _ref$leave = _ref.leave;

  var _leave = _ref$leave === undefined ? emptyFn : _ref$leave;

  var scopeManager = (0, _escope.analyze)(ast, { ecmaVersion: 6 });
  var currentScope = scopeManager.acquire(ast);

  return {
    enter: function enter(node, parent) {
      if ((0, _functionType.isFunction)(node)) {
        currentScope = scopeManager.acquire(node);
      }
      return _enter.call(this, node, parent, currentScope);
    },
    leave: function leave(node, parent) {
      if ((0, _functionType.isFunction)(node)) {
        currentScope = scopeManager.upper;
      }
      return _leave.call(this, node, parent, currentScope);
    }
  };
}
exports.default = withScope;
module.exports = exports['default'];