/* A map of tests which we know Parsoid currently fails.
 *
 * New patches which fix previously-broken tests should also patch this
 * file to document which tests are now expected to succeed.
 *
 * This helps clean up 'npm test' output, documents known bugs, and helps
 * Jenkins make sense of the parserTest output.
 *
 * NOTE that the selser blacklist depends on tests/selser.changes.json
 * If the selser change list is modified, this blacklist should be refreshed.
 *
 * This blacklist can be automagically updated by running
 *    parserTests.js --rewrite-blacklist
 * You might want to do this after you fix some bug that makes more tests
 * pass.  It is still your responsibility to carefully review the blacklist
 * diff to ensure there are no unexpected new failures (lines added).
 */

/*
 * This should map test titles to an array of test types (wt2html, wt2wt,
 * html2html, html2wt, selser) which are known to fail.
 * For easier maintenance, we group each test type together, and use a
 * helper function to create the array if needed then append the test type.
 */

'use strict';

var testBlackList = {};
var add = function(testtype, title, raw) {  // eslint-disable-line
	if (typeof (testBlackList[title]) !== 'object') {
		testBlackList[title] = {
			modes: [],
			raw: raw,
		};
	}
	testBlackList[title].modes.push(testtype);
};

// ### DO NOT REMOVE THIS LINE ### (start of automatically-generated section)

// Blacklist for wt2html
add("wt2html", "Simple <ref>, no <references/>", "<p data-parsoid='{\"dsr\":[0,58,0,0]}'>Wikipedia rocks!<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[16,58,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt3\" data-parsoid='{\"dsr\":[58,58,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">Proceeds of Rockology, vol. XXI</span></li></ol>");
add("wt2html", "Simple <ref>, with <references/>", "<p data-parsoid='{\"dsr\":[0,58,0,0]}'>Wikipedia rocks!<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[16,58,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[60,73,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">Proceeds of Rockology, vol. XXI</span></li></ol>");
add("wt2html", "<ref> with a simple template", "<p data-parsoid='{\"dsr\":[0,40,0,0]}'>Templating<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[10,40,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt5\" data-parsoid='{\"dsr\":[42,55,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\"><span about=\"#mwt3\" typeof=\"mw:Transclusion\" data-parsoid='{\"pi\":[[]],\"dsr\":[15,34,null,null]}' data-mw='{\"parts\":[{\"template\":{\"target\":{\"wt\":\"simple template\",\"href\":\"./Template:Simple_template\"},\"params\":{},\"i\":0}}]}'>A </span><i about=\"#mwt3\" data-parsoid=\"{}\">simple</i><span about=\"#mwt3\" data-parsoid=\"{}\"> template.</span></span></li></ol>");
add("wt2html", "<ref> with a <nowiki>", "<p data-parsoid='{\"dsr\":[0,57,0,0]}'>Templating<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[10,57,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt5\" data-parsoid='{\"dsr\":[59,72,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">{{simple template}}</span></li></ol>");
add("wt2html", "<ref> in a <nowiki>", "<p data-parsoid='{\"dsr\":[0,57,0,0]}'>Templating<span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[10,57,8,9]}'>&lt;ref>{{simple template}}&lt;/ref></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt3\" data-parsoid='{\"dsr\":[59,72,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'></ol>");
add("wt2html", "<ref> in a <!--comment-->", "<p data-parsoid='{\"dsr\":[0,10,0,0]}'>Templating</p><!--<ref&#x3E;{{simple template}}</ref&#x3E;-->\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt2\" data-parsoid='{\"dsr\":[49,62,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'></ol>");
add("wt2html", "<!--comment--> in a <ref> (bug 5384)", "<p data-parsoid='{\"dsr\":[0,39,0,0]}'>Templating<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[10,39,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[41,54,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">Text<!--comment--></span></li></ol>");
add("wt2html", "<references> after <gallery> (bug 6164)", "<p data-parsoid='{\"dsr\":[0,14,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,14,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ul class=\"gallery mw-gallery-traditional\" typeof=\"mw:Extension/gallery\" about=\"#mwt6\" data-parsoid='{\"dsr\":[16,51,2,2]}' data-mw='{\"name\":\"gallery\",\"attrs\":{},\"body\":{\"extsrc\":\"Image:Foobar.jpg\"}}'>\n<li class=\"gallerybox\" style=\"width: 155px;\"><div class=\"thumb\" style=\"width: 150px; height: 150px;\"><span typeof=\"mw:Image\"><a href=\"./File:Foobar.jpg\"><img resource=\"./File:Foobar.jpg\" src=\"//example.com/images/thumb/3/3a/Foobar.jpg/120px-Foobar.jpg\" data-file-width=\"1941\" data-file-height=\"220\" data-file-type=\"bitmap\" height=\"14\" width=\"120\"/></a></span></div><div class=\"gallerytext\"></div></li>\n</ul>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt5\" data-parsoid='{\"dsr\":[53,66,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">one</span></li></ol>");
add("wt2html", "{{REVISIONID}} on page with <ref> (bug 6299)", "<p data-parsoid='{\"dsr\":[0,30,0,0]}'><span about=\"#mwt2\" typeof=\"mw:Transclusion\" data-parsoid='{\"pi\":[[]],\"dsr\":[0,14,null,null]}' data-mw='{\"parts\":[{\"template\":{\"target\":{\"wt\":\"REVISIONID\",\"function\":\"revisionid\"},\"params\":{},\"i\":0}}]}'>Parser function implementation for pf_revisionid missing in Parsoid.</span><span about=\"#mwt3\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[14,30,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[30,30,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">elite</span></li></ol>");
add("wt2html", "{{REVISIONID}} on page without <ref> (bug 6299 sanity check)", "<p about=\"#mwt1\" typeof=\"mw:Transclusion\" data-parsoid='{\"dsr\":[0,14,0,0],\"pi\":[[]]}' data-mw='{\"parts\":[{\"template\":{\"target\":{\"wt\":\"REVISIONID\",\"function\":\"revisionid\"},\"params\":{},\"i\":0}}]}'>Parser function implementation for pf_revisionid missing in Parsoid.</p>");
add("wt2html", "Ref with content followed by blank ref", "<p data-parsoid='{\"dsr\":[0,31,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-blank_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,31,18,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-blank-1\"},\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[33,52,0,0]}'><span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-blank_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[33,52,19,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[54,67,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-blank-1\" id=\"cite_note-blank-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-blank_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-blank_1-1\"><span class=\"mw-linkback-text\">2 </span></a></span> <span id=\"mw-reference-text-cite_note-blank-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">content</span></li></ol>");
add("wt2html", "Blank ref followed by ref with content", "<p data-parsoid='{\"dsr\":[0,19,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-blank_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,19,19,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[21,52,0,0]}'><span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-blank_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[21,52,18,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-blank-1\"},\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[54,67,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-blank-1\" id=\"cite_note-blank-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-blank_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-blank_1-1\"><span class=\"mw-linkback-text\">2 </span></a></span> <span id=\"mw-reference-text-cite_note-blank-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">content</span></li></ol>");
add("wt2html", "Regression: non-blank ref \"0\" followed by ref with content", "<p data-parsoid='{\"dsr\":[0,25,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-blank_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,25,18,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-blank-1\"},\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[27,58,0,0]}'><span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-blank_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[27,58,18,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"html\":\"content\"},\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[60,73,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-blank-1\" id=\"cite_note-blank-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-blank_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-blank_1-1\"><span class=\"mw-linkback-text\">2 </span></a></span> <span id=\"mw-reference-text-cite_note-blank-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">0</span></li></ol>");
add("wt2html", "Regression sanity check: non-blank ref \"1\" followed by ref with content", "<p data-parsoid='{\"dsr\":[0,25,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-blank_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,25,18,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-blank-1\"},\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[27,58,0,0]}'><span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-blank_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[27,58,18,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"html\":\"content\"},\"attrs\":{\"name\":\"blank\"}}'><a href=\"./Main_Page#cite_note-blank-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[60,73,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-blank-1\" id=\"cite_note-blank-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-blank_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-blank_1-1\"><span class=\"mw-linkback-text\">2 </span></a></span> <span id=\"mw-reference-text-cite_note-blank-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">1</span></li></ol>");
add("wt2html", "Ref names containing a number", "<p data-parsoid='{\"dsr\":[0,95,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-test123test_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,33,24,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-test123test-1\"},\"attrs\":{\"name\":\"test123test\"}}'><a href=\"./Main_Page#cite_note-test123test-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\n<span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-123test_2-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[34,63,20,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-123test-2\"},\"attrs\":{\"name\":\"123test\"}}'><a href=\"./Main_Page#cite_note-123test-2\" style=\"counter-reset: mw-Ref 2;\"><span class=\"mw-reflink-text\">[2]</span></a></span>\n<span about=\"#mwt6\" class=\"mw-ref\" id=\"cite_ref-test123_3-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[64,95,20,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-test123-3\"},\"attrs\":{\"name\":\"test123\"}}'><a href=\"./Main_Page#cite_note-test123-3\" style=\"counter-reset: mw-Ref 3;\"><span class=\"mw-reflink-text\">[3]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt8\" data-parsoid='{\"dsr\":[97,111,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-test123test-1\" id=\"cite_note-test123test-1\"><a href=\"./Main_Page#cite_ref-test123test_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-test123test-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">One</span></li><li about=\"#cite_note-123test-2\" id=\"cite_note-123test-2\"><a href=\"./Main_Page#cite_ref-123test_2-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-123test-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">Two</span></li><li about=\"#cite_note-test123-3\" id=\"cite_note-test123-3\"><a href=\"./Main_Page#cite_ref-test123_3-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-test123-3\" class=\"mw-reference-text\" data-parsoid=\"{}\">Three</span></li></ol>");
add("wt2html", "Erroneous refs", "<p data-parsoid='{\"dsr\":[0,24,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-0_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,24,14,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-0-1\"},\"attrs\":{\"name\":\"0\"}}'><a href=\"./Main_Page#cite_note-0-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[26,77,0,0]}'><span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-2\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[26,77,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-2\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-2\" style=\"counter-reset: mw-Ref 2;\"><span class=\"mw-reflink-text\">[2]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[79,86,0,0]}'><span about=\"#mwt6\" class=\"mw-ref\" id=\"cite_ref-3\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[79,86,7,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{}}'><a href=\"./Main_Page#cite_note-3\" style=\"counter-reset: mw-Ref 3;\"><span class=\"mw-reflink-text\">[3]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[88,117,0,0]}'><span about=\"#mwt8\" class=\"mw-ref\" id=\"cite_ref-bar_4-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[88,117,29,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"bar\"}}'><a href=\"./Main_Page#cite_note-bar-4\" style=\"counter-reset: mw-Ref 4;\"><span class=\"mw-reflink-text\">[4]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[119,154,0,0]}'><span about=\"#mwt10\" class=\"mw-ref\" id=\"cite_ref-blankwithnoreference_5-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[119,154,35,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"blankwithnoreference\"}}'><a href=\"./Main_Page#cite_note-blankwithnoreference-5\" style=\"counter-reset: mw-Ref 5;\"><span class=\"mw-reflink-text\">[5]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt12\" data-parsoid='{\"dsr\":[156,184,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-0-1\" id=\"cite_note-0-1\"><a href=\"./Main_Page#cite_ref-0_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-0-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">Zero</span></li><li about=\"#cite_note-2\" id=\"cite_note-2\"><a href=\"./Main_Page#cite_ref-2\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">Also zero, but differently! (Normal ref)</span></li><li about=\"#cite_note-3\" id=\"cite_note-3\"><a href=\"./Main_Page#cite_ref-3\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-3\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li><li about=\"#cite_note-bar-4\" id=\"cite_note-bar-4\"><a href=\"./Main_Page#cite_ref-bar_4-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-bar-4\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li><li about=\"#cite_note-blankwithnoreference-5\" id=\"cite_note-blankwithnoreference-5\"><a href=\"./Main_Page#cite_ref-blankwithnoreference_5-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-blankwithnoreference-5\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li></ol>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt14\" data-parsoid='{\"dsr\":[186,200,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'></ol>");
add("wt2html", "Simple <ref>, with <references/> in group", "<p data-parsoid='{\"dsr\":[0,128,0,0]}'>Wikipedia rocks!<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[16,58,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\nWikipedia rocks!<span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-2\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[75,128,16,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-2\"},\"attrs\":{\"group\":\"note\"}}'><a href=\"./Main_Page#cite_note-2\" style=\"counter-reset: mw-Ref 1;\" data-mw-group=\"note\"><span class=\"mw-reflink-text\">[note 1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[130,143,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">Proceeds of Rockology, vol. XXI</span></li></ol>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt8\" data-mw-group=\"note\" data-parsoid='{\"group\":\"note\",\"dsr\":[144,168,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"note\"}}'><li about=\"#cite_note-2\" id=\"cite_note-2\"><a href=\"./Main_Page#cite_ref-2\" data-mw-group=\"note\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">Proceeds of Rockology, vol. XXI</span></li></ol>");
add("wt2html", "Simple <ref>, with <references/> in group, with groupname in Chinese", "<p data-parsoid='{\"dsr\":[0,88,0,0]}'>AAA<span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[3,29,15,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{\"group\":\"参\"}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\" data-mw-group=\"参\"><span class=\"mw-reflink-text\">[参 1]</span></a></span>BBB<span about=\"#mwt5\" class=\"mw-ref\" id=\"cite_ref-2\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[32,59,15,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-2\"},\"attrs\":{\"group\":\"注\"}}'><a href=\"./Main_Page#cite_note-2\" style=\"counter-reset: mw-Ref 1;\" data-mw-group=\"注\"><span class=\"mw-reflink-text\">[注 1]</span></a></span>CCC<span about=\"#mwt6\" class=\"mw-ref\" id=\"cite_ref-3\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[62,88,15,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-3\"},\"attrs\":{\"group\":\"参\"}}'><a href=\"./Main_Page#cite_note-3\" style=\"counter-reset: mw-Ref 2;\" data-mw-group=\"参\"><span class=\"mw-reflink-text\">[参 2]</span></a></span></p>\n\n<dl data-parsoid='{\"dsr\":[90,95,0,0]}'><dt data-parsoid='{\"dsr\":[90,95,1,0]}'>refs</dt></dl>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt8\" data-mw-group=\"参\" data-parsoid='{\"group\":\"参\",\"dsr\":[96,120,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"参\"}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" data-mw-group=\"参\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">ref a</span></li><li about=\"#cite_note-3\" id=\"cite_note-3\"><a href=\"./Main_Page#cite_ref-3\" data-mw-group=\"参\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-3\" class=\"mw-reference-text\" data-parsoid=\"{}\">ref c</span></li></ol>\n<dl data-parsoid='{\"dsr\":[121,127,0,0]}'><dt data-parsoid='{\"dsr\":[121,127,1,0]}'>notes</dt></dl>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt10\" data-mw-group=\"注\" data-parsoid='{\"group\":\"注\",\"dsr\":[128,152,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"注\"}}'><li about=\"#cite_note-2\" id=\"cite_note-2\"><a href=\"./Main_Page#cite_ref-2\" data-mw-group=\"注\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">note b</span></li></ol>");
add("wt2html", "<ref> defined in <references>", "<p data-parsoid='{\"dsr\":[0,17,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-foo_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,17,17,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"foo\"}}'><a href=\"./Main_Page#cite_note-foo-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[19,71,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"body\":{\"html\":\"\\n&lt;span about=\\\"#mwt6\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[32,57,16,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"id\\\":\\\"mw-reference-text-cite_note-foo-1\\\"},\\\"attrs\\\":{\\\"name\\\":\\\"foo\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-foo-1\\\" style=\\\"counter-reset: mw-Ref 1;\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[1]&lt;/span>&lt;/a>&lt;/span>\\n\"}}'><li about=\"#cite_note-foo-1\" id=\"cite_note-foo-1\"><a href=\"./Main_Page#cite_ref-foo_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-foo-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">BAR</span></li></ol>");
add("wt2html", "<ref> defined in <references> called with #tag", "<p data-parsoid='{\"dsr\":[0,17,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-foo_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,17,17,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"foo\"}}'><a href=\"./Main_Page#cite_note-foo-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[19,66,0,0]}'><references about=\"#mwt4\" typeof=\"mw:Transclusion\" data-parsoid='{\"dsr\":[19,66,null,null],\"pi\":[[{\"k\":\"1\"}]]}' data-mw='{\"parts\":[{\"template\":{\"target\":{\"wt\":\"#tag:references\",\"function\":\"tag\"},\"params\":{\"1\":{\"wt\":\"\\n&lt;ref name=\\\"foo\\\">BAR&lt;/ref>\\n\"}},\"i\":0}}]}'>\n<span about=\"#mwt5\" class=\"mw-ref\" id=\"cite_ref-foo_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-foo-1\"},\"attrs\":{\"name\":\"foo\"}}'><a href=\"./Main_Page#cite_note-foo-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\n</references></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[66,66,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-foo-1\" id=\"cite_note-foo-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-foo_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-foo_1-1\"><span class=\"mw-linkback-text\">2 </span></a></span> <span id=\"mw-reference-text-cite_note-foo-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">BAR</span></li></ol>");
add("wt2html", "<ref> defined in <references> error conditions", "<p data-parsoid='{\"dsr\":[0,27,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-foo_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,27,27,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"group\":\"2\",\"name\":\"foo\"}}'><a href=\"./Main_Page#cite_note-foo-1\" style=\"counter-reset: mw-Ref 1;\" data-mw-group=\"2\"><span class=\"mw-reflink-text\">[2 1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-mw-group=\"2\" data-parsoid='{\"group\":\"2\",\"dsr\":[29,173,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"2\"},\"body\":{\"html\":\"\\n&lt;span about=\\\"#mwt6\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[52,69,17,0]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"attrs\\\":{\\\"name\\\":\\\"foo\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-foo-1\\\" style=\\\"counter-reset: mw-Ref 1;\\\" data-mw-group=\\\"2\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[2 1]&lt;/span>&lt;/a>&lt;/span>\\n&lt;span about=\\\"#mwt8\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[70,98,19,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"id\\\":\\\"mw-reference-text-cite_note-unused-2\\\"},\\\"attrs\\\":{\\\"name\\\":\\\"unused\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-unused-2\\\" style=\\\"counter-reset: mw-Ref 2;\\\" data-mw-group=\\\"2\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[2 2]&lt;/span>&lt;/a>&lt;/span>\\n&lt;span about=\\\"#mwt10\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[99,140,26,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"id\\\":\\\"mw-reference-text-cite_note-foo-3\\\"},\\\"attrs\\\":{\\\"group\\\":\\\"1\\\",\\\"name\\\":\\\"foo\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-foo-3\\\" style=\\\"counter-reset: mw-Ref 1;\\\" data-mw-group=\\\"1\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[1 1]&lt;/span>&lt;/a>&lt;/span>\\n&lt;span about=\\\"#mwt12\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[141,159,5,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"id\\\":\\\"mw-reference-text-cite_note-4\\\"},\\\"attrs\\\":{}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-4\\\" style=\\\"counter-reset: mw-Ref 3;\\\" data-mw-group=\\\"2\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[2 3]&lt;/span>&lt;/a>&lt;/span>\\n\"}}'><li about=\"#cite_note-foo-1\" id=\"cite_note-foo-1\"><a href=\"./Main_Page#cite_ref-foo_1-0\" data-mw-group=\"2\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-foo-1\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li><li about=\"#cite_note-unused-2\" id=\"cite_note-unused-2\"><span rel=\"mw:referencedBy\"></span> <span id=\"mw-reference-text-cite_note-unused-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">BAR</span></li><li about=\"#cite_note-4\" id=\"cite_note-4\"><span rel=\"mw:referencedBy\"></span> <span id=\"mw-reference-text-cite_note-4\" class=\"mw-reference-text\" data-parsoid=\"{}\">BAR BAR</span></li></ol>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt13\" data-mw-group=\"1\" data-parsoid='{\"dsr\":[173,173,0,0],\"group\":\"1\"}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"1\"},\"autoGenerated\":true}'><li about=\"#cite_note-foo-3\" id=\"cite_note-foo-3\"><span rel=\"mw:referencedBy\"></span> <span id=\"mw-reference-text-cite_note-foo-3\" class=\"mw-reference-text\" data-parsoid=\"{}\">bad group</span></li></ol>");
add("wt2html", "<ref> with custom group link with number names in Klingon", "<p data-parsoid='{\"dsr\":[0,74,0,0]}'>Wikipedia rocks!<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[16,74,21,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{\"group\":\"klingon\"}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\" data-mw-group=\"klingon\"><span class=\"mw-reflink-text\">[klingon 1]</span></a></span></p>\n\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-mw-group=\"klingon\" data-parsoid='{\"group\":\"klingon\",\"dsr\":[76,105,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"klingon\"}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" data-mw-group=\"klingon\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">Proceeds of Rockology, vol. XXI</span></li></ol>");
add("wt2html", "Bug 31374 regression check: nested strip items", "<p data-parsoid='{\"dsr\":[0,48,0,0]}'><ref about=\"#mwt2\" typeof=\"mw:Transclusion\" data-parsoid='{\"a\":{\"group\":null},\"sa\":{\"group\":\"Note\"},\"dsr\":[0,48,null,null],\"pi\":[[{\"k\":\"1\"},{\"k\":\"group\",\"named\":true}]]}' data-mw='{\"parts\":[{\"template\":{\"target\":{\"wt\":\"#tag:ref\",\"function\":\"tag\"},\"params\":{\"1\":{\"wt\":\"note&lt;ref>reference&lt;/ref>\"},\"group\":{\"wt\":\"Note\"}},\"i\":0}}]}'>note<span about=\"#mwt3\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></ref></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt5\" data-mw-group=\"Note\" data-parsoid='{\"group\":\"Note\",\"dsr\":[49,74,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"Note\"}}'></ol>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt7\" data-parsoid='{\"dsr\":[75,89,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">reference</span></li></ol>");
add("wt2html", "Bug 13073 regression check: wrapped <references>", "<p data-parsoid='{\"dsr\":[0,16,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,16,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<div data-parsoid='{\"stx\":\"html\",\"dsr\":[17,42,5,6]}'><ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[22,36,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">foo\n</span></li></ol></div>");
add("wt2html", "<ref> with no name and no content.", "<p data-parsoid='{\"dsr\":[0,15,0,0]}'>Bla.<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[4,15,5,6]}' data-mw='{\"name\":\"ref\",\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt3\" data-parsoid='{\"dsr\":[15,15,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li></ol>");
add("wt2html", "<ref> with an empty-string name parameter and no content.", "<p data-parsoid='{\"dsr\":[0,23,0,0]}'>Bla.<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[4,23,13,6]}' data-mw='{\"name\":\"ref\",\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt3\" data-parsoid='{\"dsr\":[23,23,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li></ol>");
add("wt2html", "<ref> with a non-empty name parameter and no content.", "<p data-parsoid='{\"dsr\":[0,27,0,0]}'>Bla.<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-void_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[4,27,17,6]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"void\"}}'><a href=\"./Main_Page#cite_note-void-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt3\" data-parsoid='{\"dsr\":[27,27,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-void-1\" id=\"cite_note-void-1\"><a href=\"./Main_Page#cite_ref-void_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-void-1\" class=\"mw-reference-text\" data-parsoid=\"{}\"></span></li></ol>");
add("wt2html", "<ref>s with the follow parameter", "<p data-parsoid='{\"dsr\":[0,62,0,0]}'>Page one.<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-beginning_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[9,62,22,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-beginning-1\"},\"attrs\":{\"name\":\"beginning\"}}'><a href=\"./Main_Page#cite_note-beginning-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n\n<p data-parsoid='{\"dsr\":[64,129,0,0]}'>Page two.<span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-2\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[73,129,24,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-2\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-2\" style=\"counter-reset: mw-Ref 2;\"><span class=\"mw-reflink-text\">[2]</span></a></span></p>\n\n<h2 id=\"References\" data-parsoid='{\"dsr\":[131,147,2,2]}'> References </h2>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[148,162,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-beginning-1\" id=\"cite_note-beginning-1\"><a href=\"./Main_Page#cite_ref-beginning_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-beginning-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">First page footnote text.</span></li><li about=\"#cite_note-2\" id=\"cite_note-2\"><a href=\"./Main_Page#cite_ref-2\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">Second page footnote text.</span></li></ol>");
add("wt2html", "<ref> with both name and follow parameters - invalid", "<p data-parsoid='{\"dsr\":[0,74,0,0]}'>Page one.<span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-the-name_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[9,74,39,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-the-name-1\"},\"attrs\":{\"name\":\"the-name\"}}'><a href=\"./Main_Page#cite_note-the-name-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[75,89,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-the-name-1\" id=\"cite_note-the-name-1\"><a href=\"./Main_Page#cite_ref-the-name_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-the-name-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">This ref is invalid.</span></li></ol>");
add("wt2html", "Multiple definition (outside <references/>)", "<p data-parsoid='{\"dsr\":[0,43,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-a_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,21,12,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-a-1\"},\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\n<span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-a_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[22,43,12,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"html\":\"def\"},\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[44,58,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-a-1\" id=\"cite_note-a-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-a_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-a_1-1\"><span class=\"mw-linkback-text\">2 </span></a></span> <span id=\"mw-reference-text-cite_note-a-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">abc</span></li></ol>");
add("wt2html", "Multiple definition (inside <references/>)", "<p data-parsoid='{\"dsr\":[0,14,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-a_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,14,14,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[15,85,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"body\":{\"html\":\"\\n&lt;span about=\\\"#mwt6\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[28,49,12,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"id\\\":\\\"mw-reference-text-cite_note-a-1\\\"},\\\"attrs\\\":{\\\"name\\\":\\\"a\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-a-1\\\" style=\\\"counter-reset: mw-Ref 1;\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[1]&lt;/span>&lt;/a>&lt;/span>\\n&lt;span about=\\\"#mwt8\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[50,71,12,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"html\\\":\\\"def\\\"},\\\"attrs\\\":{\\\"name\\\":\\\"a\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-a-1\\\" style=\\\"counter-reset: mw-Ref 1;\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[1]&lt;/span>&lt;/a>&lt;/span>\\n\"}}'><li about=\"#cite_note-a-1\" id=\"cite_note-a-1\"><a href=\"./Main_Page#cite_ref-a_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-a-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">abc</span></li></ol>");
add("wt2html", "Multiple definition (mixed outside/inside)", "<p data-parsoid='{\"dsr\":[0,21,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-a_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,21,12,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-a-1\"},\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt4\" data-parsoid='{\"dsr\":[22,70,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"body\":{\"html\":\"\\n&lt;span about=\\\"#mwt6\\\" class=\\\"mw-ref\\\" rel=\\\"dc:references\\\" typeof=\\\"mw:Extension/ref\\\" data-parsoid=&#39;{\\\"dsr\\\":[35,56,12,6]}&#39; data-mw=&#39;{\\\"name\\\":\\\"ref\\\",\\\"body\\\":{\\\"html\\\":\\\"def\\\"},\\\"attrs\\\":{\\\"name\\\":\\\"a\\\"}}&#39;>&lt;a href=\\\"./Main_Page#cite_note-a-1\\\" style=\\\"counter-reset: mw-Ref 1;\\\" data-parsoid=\\\"{}\\\">&lt;span class=\\\"mw-reflink-text\\\" data-parsoid=\\\"{}\\\">[1]&lt;/span>&lt;/a>&lt;/span>\\n\"}}'><li about=\"#cite_note-a-1\" id=\"cite_note-a-1\"><a href=\"./Main_Page#cite_ref-a_1-0\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-a-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">abc</span></li></ol>");
add("wt2html", "Multiple definition (inside {{#tag:references}})", "<p data-parsoid='{\"dsr\":[0,80,0,0]}'><span about=\"#mwt2\" class=\"mw-ref\" id=\"cite_ref-a_1-0\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[0,14,14,0]}' data-mw='{\"name\":\"ref\",\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\n<references about=\"#mwt5\" typeof=\"mw:Transclusion\" data-parsoid='{\"dsr\":[15,80,null,null],\"pi\":[[{\"k\":\"1\"}]]}' data-mw='{\"parts\":[{\"template\":{\"target\":{\"wt\":\"#tag:references\",\"function\":\"tag\"},\"params\":{\"1\":{\"wt\":\"\\n&lt;ref name=a>abc&lt;/ref>\\n&lt;ref name=a>def&lt;/ref>\\n\"}},\"i\":0}}]}'>\n<span about=\"#mwt6\" class=\"mw-ref\" id=\"cite_ref-a_1-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-a-1\"},\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\n<span about=\"#mwt7\" class=\"mw-ref\" id=\"cite_ref-a_1-2\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-mw='{\"name\":\"ref\",\"body\":{\"html\":\"def\"},\"attrs\":{\"name\":\"a\"}}'><a href=\"./Main_Page#cite_note-a-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span>\n</references></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt8\" data-parsoid='{\"dsr\":[80,80,0,0]}' data-mw='{\"name\":\"references\",\"attrs\":{},\"autoGenerated\":true}'><li about=\"#cite_note-a-1\" id=\"cite_note-a-1\"><span rel=\"mw:referencedBy\"><a href=\"./Main_Page#cite_ref-a_1-0\"><span class=\"mw-linkback-text\">1 </span></a><a href=\"./Main_Page#cite_ref-a_1-1\"><span class=\"mw-linkback-text\">2 </span></a><a href=\"./Main_Page#cite_ref-a_1-2\"><span class=\"mw-linkback-text\">3 </span></a></span> <span id=\"mw-reference-text-cite_note-a-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">abc</span></li></ol>");


// Blacklist for wt2wt
add("wt2wt", "Simple <ref>, no <references/>", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n<references />");
add("wt2wt", "Simple <ref>, with <references/>", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n<references />");
add("wt2wt", "<ref> with a simple template", "Templating<ref>{{simple template}}</ref>\n\n<references />");
add("wt2wt", "<ref> with a <nowiki>", "Templating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n<references />");
add("wt2wt", "<ref> in a <nowiki>", "Templating<nowiki><ref>{{simple template}}</ref></nowiki>\n\n<references />");
add("wt2wt", "<ref> in a <!--comment-->", "Templating<!--<ref>{{simple template}}</ref>-->\n\n<references />");
add("wt2wt", "<!--comment--> in a <ref> (bug 5384)", "Templating<ref>Text<!--comment--></ref>\n\n<references />");
add("wt2wt", "<references> after <gallery> (bug 6164)", "<ref>one</ref>\n\n<gallery>Image:Foobar.jpg</gallery>\n\n<references />");
add("wt2wt", "{{REVISIONID}} on page with <ref> (bug 6299)", "{{REVISIONID}}<ref>elite</ref>\n<references />");
add("wt2wt", "Ref with content followed by blank ref", "<ref name=\"blank\">content</ref>\n\n<ref name=\"blank\" />\n\n<references />");
add("wt2wt", "Blank ref followed by ref with content", "<ref name=\"blank\" />\n\n<ref name=\"blank\">content</ref>\n\n<references />");
add("wt2wt", "Regression: non-blank ref \"0\" followed by ref with content", "<ref name=\"blank\">0</ref>\n\n<ref name=\"blank\">content</ref>\n\n<references />");
add("wt2wt", "Regression sanity check: non-blank ref \"1\" followed by ref with content", "<ref name=\"blank\">1</ref>\n\n<ref name=\"blank\">content</ref>\n\n<references />");
add("wt2wt", "Erroneous refs", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\n<ref name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n<references />\n\n<references />");
add("wt2wt", "Simple <ref>, with <references/> in group", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=\"note\">Proceeds of Rockology, vol. XXI</ref>\n\n<references />\n<references group=\"note\" />");
add("wt2wt", "<ref> defined in <references>", "<ref name=\"foo\" />\n\n<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("wt2wt", "<ref> defined in <references> called with #tag", "<ref name=\"foo\" />\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}\n<references />");
add("wt2wt", "<ref> defined in <references> error conditions", "<ref group=\"2\" name=\"foo\" />\n\n<references group=\"2\">\n<ref name=\"foo\" />\n<ref name=\"unused\">BAR</ref>\n<ref group=\"1\" name=\"foo\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>\n<references group=\"1\" />");
add("wt2wt", "<ref> with custom group link with number names in Klingon", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n<references group=\"klingon\" />");
add("wt2wt", "Bug 31374 regression check: nested strip items", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n<references group=\"Note\" />\n<references />");
add("wt2wt", "Bug 13073 regression check: wrapped <references>", "<ref>foo\n</ref>\n<div><references /></div>");
add("wt2wt", "<ref> with no name and no content.", "Bla.<ref />\n<references />");
add("wt2wt", "<ref> with an empty-string name parameter and no content.", "Bla.<ref />\n<references />");
add("wt2wt", "<ref> with a non-empty name parameter and no content.", "Bla.<ref name=\"void\" />\n<references />");
add("wt2wt", "<ref>s with the follow parameter", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref>Second page footnote text.</ref>\n\n== References ==\n<references />");
add("wt2wt", "<ref> with both name and follow parameters - invalid", "Page one.<ref name=\"the-name\">This ref is invalid.</ref>\n<references />");
add("wt2wt", "Multiple definition (outside <references/>)", "<ref name=\"a\">abc</ref>\n<ref name=\"a\">def</ref>\n<references />");
add("wt2wt", "Multiple definition (inside <references/>)", "<ref name=\"a\" />\n<references>\n<ref name=\"a\">abc</ref>\n<ref name=\"a\">def</ref>\n</references>");
add("wt2wt", "Multiple definition (mixed outside/inside)", "<ref name=\"a\">abc</ref>\n<references>\n<ref name=\"a\">def</ref>\n</references>");
add("wt2wt", "Multiple definition (inside {{#tag:references}})", "<ref name=\"a\" />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n<references />");
add("wt2wt", "Ref: 3. spaces in ref-names should be ignored", "A <ref name=\"x\">foo</ref>\nB <ref name=\"x\" />\nC <ref name=\"x\" />\n<references />");
add("wt2wt", "Ref: 5. body should accept generic wikitext", "A <ref>This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("wt2wt", "Ref: 6. indent-pres should not be output in ref-body", "A <ref>foo\n bar\n baz\n</ref>\n\n<references />");
add("wt2wt", "Ref: 7. No p-wrapping in ref-body", "A <ref>foo\nbar\nbaz\nbooz\n</ref>\n\n<references />");
add("wt2wt", "Ref: 8. transclusion wikitext has lower precedence", "A <ref><nowiki>foo {{echo|</nowiki></ref><nowiki> B C}}</nowiki>\n\n<references />");
add("wt2wt", "Ref: 9. unclosed comments should not leak out of ref-body", "A <ref>foo <!----></ref> B C\n<references />");
add("wt2wt", "Ref: 10. Unclosed HTML tags should not leak out of ref-body", "A <ref><b> foo </ref> B C\n\n<references />");
add("wt2wt", "References: 9. Generate missing references list at the end", "A <ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>\n<references />\n<references group=\"inexistent\" />");


// Blacklist for html2html
add("html2html", "Simple <ref>, no <references/>", "<p data-parsoid='{\"dsr\":[0,81,0,0]}'>Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[16,81,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[55,75,15,2]}'>[1]</a></sup></p>\n<ol data-parsoid='{\"dsr\":[82,207,0,0]}'><li data-parsoid='{\"dsr\":[82,207,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[84,139,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[115,132,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[140,207,29,7]}'>Proceeds of Rockology, vol. XXI</span></li></ol>\n");
add("html2html", "Simple <ref>, with <references/>", "<p data-parsoid='{\"dsr\":[0,81,0,0]}'>Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[16,81,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[55,75,15,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[83,208,0,0]}'><li data-parsoid='{\"dsr\":[83,208,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[85,140,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[116,133,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[141,208,29,7]}'>Proceeds of Rockology, vol. XXI</span></li></ol>\n");
add("html2html", "<ref> with a simple template", "<p data-parsoid='{\"dsr\":[0,75,0,0]}'>Templating<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[10,75,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[49,69,15,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[77,193,0,0]}'><li data-parsoid='{\"dsr\":[77,193,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[79,134,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[110,127,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[135,193,29,7]}'>A <i data-parsoid='{\"dsr\":[166,176,2,2]}'>simple</i> template.</span></li></ol>\n");
add("html2html", "<ref> with a <nowiki>", "<p data-parsoid='{\"dsr\":[0,75,0,0]}'>Templating<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[10,75,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[49,69,15,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[77,207,0,0]}'><li data-parsoid='{\"dsr\":[77,207,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[79,134,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[110,127,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[135,207,29,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[164,200,8,9]}'>{{simple template}}</span></span></li></ol>\n");
add("html2html", "<!--comment--> in a <ref> (bug 5384)", "<p data-parsoid='{\"dsr\":[0,75,0,0]}'>Templating<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[10,75,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[49,69,15,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[77,175,0,0]}'><li data-parsoid='{\"dsr\":[77,175,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[79,134,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[110,127,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[135,175,29,7]}'>Text</span></li></ol>\n");
add("html2html", "<references> after <gallery> (bug 6164)", "<p data-parsoid='{\"dsr\":[0,65,0,0]}'><sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,65,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[39,59,15,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[67,71,0,0]}'>\t\t* </p><div style=\"width: 155px\" data-parsoid='{\"stx\":\"html\",\"dsr\":[71,296,26,6]}'> \t\t\t<div class=\"thumb\" style=\"width: 150px;\" data-parsoid='{\"stx\":\"html\",\"dsr\":[101,249,41,6]}'><div style=\"margin:68px auto;\" data-parsoid='{\"stx\":\"html\",\"dsr\":[142,243,31,6]}'><img src=\"http://example.com/images/thumb/3/3a/Foobar.jpg/120px-Foobar.jpg\" alt=\"120px-Foobar.jpg\" rel=\"mw:externalImage\" data-parsoid='{\"dsr\":[173,237,null,null]}'/></div></div> \t\t\t<div class=\"gallerytext\" data-parsoid='{\"stx\":\"html\",\"dsr\":[253,287,25,6]}'>\t\t\t</div> \t\t</div>\n\n<ol data-parsoid='{\"dsr\":[298,395,0,0]}'><li data-parsoid='{\"dsr\":[298,395,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[300,355,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[331,348,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[356,395,29,7]}'>one</span></li></ol>\n");
add("html2html", "{{REVISIONID}} on page with <ref> (bug 6299)", "<p data-parsoid='{\"dsr\":[0,69,0,0]}'>1337<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[4,69,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[43,63,15,2]}'>[1]</a></sup></p>\n<ol data-parsoid='{\"dsr\":[70,169,0,0]}'><li data-parsoid='{\"dsr\":[70,169,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[72,127,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[103,120,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[128,169,29,7]}'>elite</span></li></ol>\n");
add("html2html", "Ref with content followed by blank ref", "<p data-parsoid='{\"dsr\":[0,79,0,0]}'><sup id=\"cite_ref-blank_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,79,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[47,73,21,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[81,160,0,0]}'><sup id=\"cite_ref-blank_1-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[81,160,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[128,154,21,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[162,325,0,0]}'><li data-parsoid='{\"dsr\":[162,325,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[164,281,31,7]}'>↑ <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[197,235,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-0\"},\"sa\":{\"href\":\"#cite_ref-blank_1-0\"},\"dsr\":[202,229,22,2]}'>1.0</a></sup> <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[236,274,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-1\"},\"sa\":{\"href\":\"#cite_ref-blank_1-1\"},\"dsr\":[241,268,22,2]}'>1.1</a></sup></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[282,325,29,7]}'>content</span></li></ol>\n");
add("html2html", "Blank ref followed by ref with content", "<p data-parsoid='{\"dsr\":[0,79,0,0]}'><sup id=\"cite_ref-blank_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,79,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[47,73,21,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[81,160,0,0]}'><sup id=\"cite_ref-blank_1-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[81,160,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[128,154,21,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[162,325,0,0]}'><li data-parsoid='{\"dsr\":[162,325,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[164,281,31,7]}'>↑ <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[197,235,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-0\"},\"sa\":{\"href\":\"#cite_ref-blank_1-0\"},\"dsr\":[202,229,22,2]}'>1.0</a></sup> <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[236,274,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-1\"},\"sa\":{\"href\":\"#cite_ref-blank_1-1\"},\"dsr\":[241,268,22,2]}'>1.1</a></sup></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[282,325,29,7]}'>content</span></li></ol>\n");
add("html2html", "Regression: non-blank ref \"0\" followed by ref with content", "<p data-parsoid='{\"dsr\":[0,79,0,0]}'><sup id=\"cite_ref-blank_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,79,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[47,73,21,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[81,160,0,0]}'><sup id=\"cite_ref-blank_1-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[81,160,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[128,154,21,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[162,504,0,0]}'><li data-parsoid='{\"dsr\":[162,504,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[164,281,31,7]}'>↑ <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[197,235,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-0\"},\"sa\":{\"href\":\"#cite_ref-blank_1-0\"},\"dsr\":[202,229,22,2]}'>1.0</a></sup> <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[236,274,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-1\"},\"sa\":{\"href\":\"#cite_ref-blank_1-1\"},\"dsr\":[241,268,22,2]}'>1.1</a></sup></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[282,504,29,7]}'>0 <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[313,497,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[391,426,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[397,419,8,9]}'>&lt;ref></span></code> tag; name \"blank\" defined multiple times with different content</span></span></li></ol>\n");
add("html2html", "Regression sanity check: non-blank ref \"1\" followed by ref with content", "<p data-parsoid='{\"dsr\":[0,79,0,0]}'><sup id=\"cite_ref-blank_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,79,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[47,73,21,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[81,160,0,0]}'><sup id=\"cite_ref-blank_1-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[81,160,47,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blank-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blank-1\"},\"sa\":{\"href\":\"#cite_note-blank-1\"},\"dsr\":[128,154,21,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[162,504,0,0]}'><li data-parsoid='{\"dsr\":[162,504,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[164,281,31,7]}'>↑ <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[197,235,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-0\"},\"sa\":{\"href\":\"#cite_ref-blank_1-0\"},\"dsr\":[202,229,22,2]}'>1.0</a></sup> <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[236,274,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blank_1-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blank_1-1\"},\"sa\":{\"href\":\"#cite_ref-blank_1-1\"},\"dsr\":[241,268,22,2]}'>1.1</a></sup></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[282,504,29,7]}'>1 <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[313,497,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[391,426,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[397,419,8,9]}'>&lt;ref></span></code> tag; name \"blank\" defined multiple times with different content</span></span></li></ol>\n");
add("html2html", "Ref names containing a number", "<p data-parsoid='{\"dsr\":[0,259,0,0]}'><sup id=\"cite_ref-test123test_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,91,53,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-test123test-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-test123test-1\"},\"sa\":{\"href\":\"#cite_note-test123test-1\"},\"dsr\":[53,85,27,2]}'>[1]</a></sup>\n<sup id=\"cite_ref-123test_2-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[92,175,49,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-123test-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-123test-2\"},\"sa\":{\"href\":\"#cite_note-123test-2\"},\"dsr\":[141,169,23,2]}'>[2]</a></sup>\n<sup id=\"cite_ref-test123_3-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[176,259,49,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-test123-3\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-test123-3\"},\"sa\":{\"href\":\"#cite_note-test123-3\"},\"dsr\":[225,253,23,2]}'>[3]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[261,590,0,0]}'><li data-parsoid='{\"dsr\":[261,372,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[263,332,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-test123test_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-test123test_1-0\"},\"sa\":{\"href\":\"#cite_ref-test123test_1-0\"},\"dsr\":[294,325,28,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[333,372,29,7]}'>One</span></li>\n<li data-parsoid='{\"dsr\":[373,480,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[375,440,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-123test_2-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-123test_2-0\"},\"sa\":{\"href\":\"#cite_ref-123test_2-0\"},\"dsr\":[406,433,24,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[441,480,29,7]}'>Two</span></li>\n<li data-parsoid='{\"dsr\":[481,590,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[483,548,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-test123_3-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-test123_3-0\"},\"sa\":{\"href\":\"#cite_ref-test123_3-0\"},\"dsr\":[514,541,24,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[549,590,29,7]}'>Three</span></li></ol>\n");
add("html2html", "Erroneous refs", "<p data-parsoid='{\"dsr\":[0,182,0,0]}'><span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,182,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[78,113,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[84,106,8,9]}'>&lt;ref></span></code> tag;\nname cannot be a simple integer. Use a descriptive title</span></p>\n\n<p data-parsoid='{\"dsr\":[184,249,0,0]}'><sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[184,249,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[223,243,15,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[251,410,0,0]}'><span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[251,410,58,7]}'>Cite error: The opening <code data-parsoid='{\"stx\":\"html\",\"dsr\":[333,368,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[339,361,8,9]}'>&lt;ref></span></code> tag is malformed or has a bad name</span></p>\n\n<p data-parsoid='{\"dsr\":[412,487,0,0]}'><sup id=\"cite_ref-bar_2-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[412,487,45,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-bar-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-bar-2\"},\"sa\":{\"href\":\"#cite_note-bar-2\"},\"dsr\":[457,481,19,2]}'>[2]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[489,598,0,0]}'><sup id=\"cite_ref-blankwithnoreference_3-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[489,598,62,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-blankwithnoreference-3\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-blankwithnoreference-3\"},\"sa\":{\"href\":\"#cite_note-blankwithnoreference-3\"},\"dsr\":[551,592,36,2]}'>[3]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[600,744,0,0]}'><span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[600,744,58,7]}'>Cite error: Invalid parameter in <code data-parsoid='{\"stx\":\"html\",\"dsr\":[691,733,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[697,726,8,9]}'>&lt;references></span></code> tag</span></p>\n\n<ol data-parsoid='{\"dsr\":[746,1400,0,0]}'><li data-parsoid='{\"dsr\":[746,880,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[748,803,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[779,796,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[804,880,29,7]}'>Also zero, but differently! (Normal ref)</span></li>\n<li data-parsoid='{\"dsr\":[881,1123,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[883,944,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-bar_2-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-bar_2-0\"},\"sa\":{\"href\":\"#cite_ref-bar_2-0\"},\"dsr\":[914,937,20,2]}'>↑</a></span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[945,1123,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[1023,1058,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[1029,1051,8,9]}'>&lt;ref></span></code> tag; no text was provided for refs named <code data-parsoid='{\"stx\":\"html\",\"dsr\":[1100,1116,6,7]}'>bar</code></span></li>\n<li data-parsoid='{\"dsr\":[1124,1400,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[1126,1204,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-blankwithnoreference_3-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-blankwithnoreference_3-0\"},\"sa\":{\"href\":\"#cite_ref-blankwithnoreference_3-0\"},\"dsr\":[1157,1197,37,2]}'>↑</a></span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[1205,1400,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[1283,1318,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[1289,1311,8,9]}'>&lt;ref></span></code> tag; no text was provided for refs named <code data-parsoid='{\"stx\":\"html\",\"dsr\":[1360,1393,6,7]}'>blankwithnoreference</code></span></li></ol>\n");
add("html2html", "Simple <ref>, with <references/> in group", "<p data-parsoid='{\"dsr\":[0,168,0,0]}'>Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[16,81,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[55,75,15,2]}'>[1]</a></sup>\nWikipedia rocks!<sup id=\"cite_ref-2\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[98,168,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-2\"},\"sa\":{\"href\":\"#cite_note-2\"},\"dsr\":[137,162,15,2]}'>[note 1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[170,295,0,0]}'><li data-parsoid='{\"dsr\":[170,295,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[172,227,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[203,220,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[228,295,29,7]}'>Proceeds of Rockology, vol. XXI</span></li></ol>\n\n<ol data-parsoid='{\"dsr\":[297,422,0,0]}'><li data-parsoid='{\"dsr\":[297,422,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[299,354,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-2\"},\"sa\":{\"href\":\"#cite_ref-2\"},\"dsr\":[330,347,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[355,422,29,7]}'>Proceeds of Rockology, vol. XXI</span></li></ol>\n");
add("html2html", "Simple <ref>, with <references/> in group, with groupname in Chinese", "<p data-parsoid='{\"dsr\":[0,210,0,0]}'>AAA<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[3,70,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[42,64,15,2]}'>[参 1]</a></sup>BBB<sup id=\"cite_ref-2\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[73,140,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-2\"},\"sa\":{\"href\":\"#cite_note-2\"},\"dsr\":[112,134,15,2]}'>[注 1]</a></sup>CCC<sup id=\"cite_ref-3\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[143,210,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-3\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-3\"},\"sa\":{\"href\":\"#cite_note-3\"},\"dsr\":[182,204,15,2]}'>[参 2]</a></sup></p>\n\n<dl data-parsoid='{\"dsr\":[212,218,0,0]}'><dt data-parsoid='{\"dsr\":[212,218,1,0]}'> refs</dt></dl>\n\n<ol data-parsoid='{\"dsr\":[220,419,0,0]}'><li data-parsoid='{\"dsr\":[220,319,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[222,277,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[253,270,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[278,319,29,7]}'>ref a</span></li>\n<li data-parsoid='{\"dsr\":[320,419,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[322,377,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-3\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-3\"},\"sa\":{\"href\":\"#cite_ref-3\"},\"dsr\":[353,370,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[378,419,29,7]}'>ref c</span></li></ol>\n\n<dl data-parsoid='{\"dsr\":[421,428,0,0]}'><dt data-parsoid='{\"dsr\":[421,428,1,0]}'> notes</dt></dl>\n\n<ol data-parsoid='{\"dsr\":[430,530,0,0]}'><li data-parsoid='{\"dsr\":[430,530,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[432,487,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-2\"},\"sa\":{\"href\":\"#cite_ref-2\"},\"dsr\":[463,480,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[488,530,29,7]}'>note b</span></li></ol>\n");
add("html2html", "<ref> defined in <references>", "<p data-parsoid='{\"dsr\":[0,75,0,0]}'><sup id=\"cite_ref-foo_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,75,45,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-foo-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-foo-1\"},\"sa\":{\"href\":\"#cite_note-foo-1\"},\"dsr\":[45,69,19,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[77,180,0,0]}'><li data-parsoid='{\"dsr\":[77,180,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[79,140,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-foo_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-foo_1-0\"},\"sa\":{\"href\":\"#cite_ref-foo_1-0\"},\"dsr\":[110,133,20,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[141,180,29,7]}'>BAR</span></li></ol>\n");
add("html2html", "<ref> defined in <references> called with #tag", "<p data-parsoid='{\"dsr\":[0,75,0,0]}'><sup id=\"cite_ref-foo_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,75,45,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-foo-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-foo-1\"},\"sa\":{\"href\":\"#cite_note-foo-1\"},\"dsr\":[45,69,19,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[77,180,0,0]}'><li data-parsoid='{\"dsr\":[77,180,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[79,140,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-foo_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-foo_1-0\"},\"sa\":{\"href\":\"#cite_ref-foo_1-0\"},\"dsr\":[110,133,20,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[141,180,29,7]}'>BAR</span></li></ol>\n");
add("html2html", "<ref> defined in <references> error conditions", "<p data-parsoid='{\"dsr\":[0,77,0,0]}'><sup id=\"cite_ref-foo_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,77,45,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-foo-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-foo-1\"},\"sa\":{\"href\":\"#cite_note-foo-1\"},\"dsr\":[45,71,19,2]}'>[2 1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[79,321,0,0]}'><li data-parsoid='{\"dsr\":[79,321,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[81,142,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-foo_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-foo_1-0\"},\"sa\":{\"href\":\"#cite_ref-foo_1-0\"},\"dsr\":[112,135,20,2]}'>↑</a></span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[143,321,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[221,256,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[227,249,8,9]}'>&lt;ref></span></code> tag; no text was provided for refs named <code data-parsoid='{\"stx\":\"html\",\"dsr\":[298,314,6,7]}'>foo</code></span></li></ol>\n\n<p data-parsoid='{\"dsr\":[323,933,0,0]}'><span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[323,539,58,7]}'>Cite error: <code data-parsoid='{\"stx\":\"html\",\"dsr\":[393,428,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[399,421,8,9]}'>&lt;ref></span></code> tag with name \"unused\" defined in <code data-parsoid='{\"stx\":\"html\",\"dsr\":[463,505,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[469,498,8,9]}'>&lt;references></span></code> is not used in prior text.</span>\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[540,739,58,7]}'>Cite error: <code data-parsoid='{\"stx\":\"html\",\"dsr\":[610,645,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[616,638,8,9]}'>&lt;ref></span></code> tag in <code data-parsoid='{\"stx\":\"html\",\"dsr\":[653,695,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[659,688,8,9]}'>&lt;references></span></code> has conflicting group attribute \"1\".</span>\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[740,933,58,7]}'>Cite error: <code data-parsoid='{\"stx\":\"html\",\"dsr\":[810,845,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[816,838,8,9]}'>&lt;ref></span></code> tag defined in <code data-parsoid='{\"stx\":\"html\",\"dsr\":[861,903,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[867,896,8,9]}'>&lt;references></span></code> has no name attribute.</span></p>\n");
add("html2html", "<ref> with custom group link with number names in Klingon", "<p data-parsoid='{\"dsr\":[0,83,0,0]}'>Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[16,83,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[55,76,15,2]}'>[wa'</a>]</sup></p>\n\n<ol data-parsoid='{\"dsr\":[85,210,0,0]}'><li data-parsoid='{\"dsr\":[85,210,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[87,142,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[118,135,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[143,210,29,7]}'>Proceeds of Rockology, vol. XXI</span></li></ol>\n");
add("html2html", "Bug 31374 regression check: nested strip items", "<p data-parsoid='{\"dsr\":[0,70,0,0]}'><sup id=\"cite_ref-2\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,70,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-2\"},\"sa\":{\"href\":\"#cite_note-2\"},\"dsr\":[39,64,15,2]}'>[Note 1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[72,235,0,0]}'><li data-parsoid='{\"dsr\":[72,235,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[74,129,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-2\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-2\"},\"sa\":{\"href\":\"#cite_ref-2\"},\"dsr\":[105,122,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[130,235,29,7]}'>note<sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[163,228,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[202,222,15,2]}'>[1]</a></sup></span></li></ol>\n\n<ol data-parsoid='{\"dsr\":[237,340,0,0]}'><li data-parsoid='{\"dsr\":[237,340,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[239,294,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[270,287,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[295,340,29,7]}'>reference</span></li></ol>\n");
add("html2html", "Bug 13073 regression check: wrapped <references>", "<p data-parsoid='{\"dsr\":[0,65,0,0]}'><sup id=\"cite_ref-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,65,39,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-1\"},\"sa\":{\"href\":\"#cite_note-1\"},\"dsr\":[39,59,15,2]}'>[1]</a></sup></p>\n\n<div data-parsoid='{\"stx\":\"html\",\"dsr\":[67,179,5,6]}'>\n<ol data-parsoid='{\"dsr\":[73,171,0,0]}'><li data-parsoid='{\"dsr\":[73,171,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[75,130,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-1\"},\"sa\":{\"href\":\"#cite_ref-1\"},\"dsr\":[106,123,14,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[131,171,29,7]}'> foo</span></li></ol>\n\n</div>\n");
add("html2html", "<ref> with a non-empty name parameter and no content.", "<p data-parsoid='{\"dsr\":[0,81,0,0]}'>Bla.<sup id=\"cite_ref-void_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[4,81,46,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-void-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-void-1\"},\"sa\":{\"href\":\"#cite_note-void-1\"},\"dsr\":[50,75,20,2]}'>[1]</a></sup></p>\n<ol data-parsoid='{\"dsr\":[82,326,0,0]}'><li data-parsoid='{\"dsr\":[82,326,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[84,146,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-void_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-void_1-0\"},\"sa\":{\"href\":\"#cite_ref-void_1-0\"},\"dsr\":[115,139,21,2]}'>↑</a></span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[147,326,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[225,260,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[231,253,8,9]}'>&lt;ref></span></code> tag; no text was provided for refs named <code data-parsoid='{\"stx\":\"html\",\"dsr\":[302,319,6,7]}'>void</code></span></li></ol>\n");
add("html2html", "<ref>s with the follow parameter", "<p data-parsoid='{\"dsr\":[0,96,0,0]}'>Page one.<sup id=\"cite_ref-beginning_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[9,96,51,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-beginning-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-beginning-1\"},\"sa\":{\"href\":\"#cite_note-beginning-1\"},\"dsr\":[60,90,25,2]}'>[1]</a></sup></p>\n\n<p data-parsoid='{\"dsr\":[98,107,0,0]}'>Page two.</p>\n\n<h2 id=\"References\" data-parsoid='{\"dsr\":[109,125,2,2]}'> References </h2>\n\n<ol data-parsoid='{\"dsr\":[127,285,0,0]}'><li data-parsoid='{\"dsr\":[127,285,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[129,196,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-beginning_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-beginning_1-0\"},\"sa\":{\"href\":\"#cite_ref-beginning_1-0\"},\"dsr\":[160,189,26,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[197,285,29,7]}'>First page footnote text. Second page footnote text.</span></li></ol>\n");
add("html2html", "Multiple definition (outside <references/>)", "<p data-parsoid='{\"dsr\":[0,143,0,0]}'><sup id=\"cite_ref-a_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,71,43,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-a-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-a-1\"},\"sa\":{\"href\":\"#cite_note-a-1\"},\"dsr\":[43,65,17,2]}'>[1]</a></sup>\n<sup id=\"cite_ref-a_1-1\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[72,143,43,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-a-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-a-1\"},\"sa\":{\"href\":\"#cite_note-a-1\"},\"dsr\":[115,137,17,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[145,477,0,0]}'><li data-parsoid='{\"dsr\":[145,477,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[147,256,31,7]}'>↑ <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[180,214,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-a_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-a_1-0\"},\"sa\":{\"href\":\"#cite_ref-a_1-0\"},\"dsr\":[185,208,18,2]}'>1.0</a></sup> <sup data-parsoid='{\"stx\":\"html\",\"dsr\":[215,249,5,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-a_1-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-a_1-1\"},\"sa\":{\"href\":\"#cite_ref-a_1-1\"},\"dsr\":[220,243,18,2]}'>1.1</a></sup></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[257,477,29,7]}'>abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[290,470,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[368,403,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[374,396,8,9]}'>&lt;ref></span></code> tag; name \"a\" defined multiple times with different content</span></span></li></ol>\n");
add("html2html", "Multiple definition (inside <references/>)", "<p data-parsoid='{\"dsr\":[0,71,0,0]}'><sup id=\"cite_ref-a_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,71,43,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-a-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-a-1\"},\"sa\":{\"href\":\"#cite_note-a-1\"},\"dsr\":[43,65,17,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[73,355,0,0]}'><li data-parsoid='{\"dsr\":[73,355,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[75,134,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-a_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-a_1-0\"},\"sa\":{\"href\":\"#cite_ref-a_1-0\"},\"dsr\":[106,127,18,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[135,355,29,7]}'>abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[168,348,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[246,281,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[252,274,8,9]}'>&lt;ref></span></code> tag; name \"a\" defined multiple times with different content</span></span></li></ol>\n");
add("html2html", "Multiple definition (mixed outside/inside)", "<p data-parsoid='{\"dsr\":[0,71,0,0]}'><sup id=\"cite_ref-a_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,71,43,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-a-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-a-1\"},\"sa\":{\"href\":\"#cite_note-a-1\"},\"dsr\":[43,65,17,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[73,355,0,0]}'><li data-parsoid='{\"dsr\":[73,355,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[75,134,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-a_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-a_1-0\"},\"sa\":{\"href\":\"#cite_ref-a_1-0\"},\"dsr\":[106,127,18,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[135,355,29,7]}'>abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[168,348,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[246,281,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[252,274,8,9]}'>&lt;ref></span></code> tag; name \"a\" defined multiple times with different content</span></span></li></ol>\n");
add("html2html", "Multiple definition (inside {{#tag:references}})", "<p data-parsoid='{\"dsr\":[0,71,0,0]}'><sup id=\"cite_ref-a_1-0\" class=\"reference\" data-parsoid='{\"stx\":\"html\",\"dsr\":[0,71,43,6]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_note-a-1\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_note-a-1\"},\"sa\":{\"href\":\"#cite_note-a-1\"},\"dsr\":[43,65,17,2]}'>[1]</a></sup></p>\n\n<ol data-parsoid='{\"dsr\":[73,355,0,0]}'><li data-parsoid='{\"dsr\":[73,355,1,0]}'> <span class=\"mw-cite-backlink\" data-parsoid='{\"stx\":\"html\",\"dsr\":[75,134,31,7]}'><a rel=\"mw:WikiLink\" href=\"./Main_Page#cite_ref-a_1-0\" data-parsoid='{\"stx\":\"piped\",\"a\":{\"href\":\"./Main_Page#cite_ref-a_1-0\"},\"sa\":{\"href\":\"#cite_ref-a_1-0\"},\"dsr\":[106,127,18,2]}'>↑</a></span> <span class=\"reference-text\" data-parsoid='{\"stx\":\"html\",\"dsr\":[135,355,29,7]}'>abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\" data-parsoid='{\"stx\":\"html\",\"dsr\":[168,348,58,7]}'>Cite error: Invalid <code data-parsoid='{\"stx\":\"html\",\"dsr\":[246,281,6,7]}'><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[252,274,8,9]}'>&lt;ref></span></code> tag; name \"a\" defined multiple times with different content</span></span></li></ol>\n");
add("html2html", "Ref: 8. transclusion wikitext has lower precedence", "<p data-parsoid='{\"dsr\":[0,64,0,0]}'>A <span about=\"#mwt3\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[2,41,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span><span typeof=\"mw:Nowiki\" data-parsoid='{\"dsr\":[41,64,8,9]}'> B C}}</span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[65,79,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">foo {{echo|</span></li></ol>");
add("html2html", "References: 9. Generate missing references list at the end", "<p data-parsoid='{\"dsr\":[0,52,0,0]}'>A <span about=\"#mwt3\" class=\"mw-ref\" id=\"cite_ref-1\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[2,16,5,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-1\"},\"attrs\":{}}'><a href=\"./Main_Page#cite_note-1\" style=\"counter-reset: mw-Ref 1;\"><span class=\"mw-reflink-text\">[1]</span></a></span> B <span about=\"#mwt4\" class=\"mw-ref\" id=\"cite_ref-2\" rel=\"dc:references\" typeof=\"mw:Extension/ref\" data-parsoid='{\"dsr\":[19,52,24,6]}' data-mw='{\"name\":\"ref\",\"body\":{\"id\":\"mw-reference-text-cite_note-2\"},\"attrs\":{\"group\":\"inexistent\"}}'><a href=\"./Main_Page#cite_note-2\" style=\"counter-reset: mw-Ref 1;\" data-mw-group=\"inexistent\"><span class=\"mw-reflink-text\">[inexistent 1]</span></a></span></p>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt6\" data-parsoid='{\"dsr\":[53,67,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{}}'><li about=\"#cite_note-1\" id=\"cite_note-1\"><a href=\"./Main_Page#cite_ref-1\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-1\" class=\"mw-reference-text\" data-parsoid=\"{}\">foo</span></li></ol>\n<ol class=\"mw-references references\" typeof=\"mw:Extension/references\" about=\"#mwt8\" data-mw-group=\"inexistent\" data-parsoid='{\"group\":\"inexistent\",\"dsr\":[68,101,2,2]}' data-mw='{\"name\":\"references\",\"attrs\":{\"group\":\"inexistent\"}}'><li about=\"#cite_note-2\" id=\"cite_note-2\"><a href=\"./Main_Page#cite_ref-2\" data-mw-group=\"inexistent\" rel=\"mw:referencedBy\"><span class=\"mw-linkback-text\">↑ </span></a> <span id=\"mw-reference-text-cite_note-2\" class=\"mw-reference-text\" data-parsoid=\"{}\">bar</span></li></ol>");


// Blacklist for html2wt
add("html2wt", "Simple <ref>, no <references/>", "Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">Proceeds of Rockology, vol. XXI</span>\n");
add("html2wt", "Simple <ref>, with <references/>", "Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">Proceeds of Rockology, vol. XXI</span>\n");
add("html2wt", "<ref> with a simple template", "Templating<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">A ''simple'' template.</span>\n");
add("html2wt", "<ref> with a <nowiki>", "Templating<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\"><nowiki>{{simple template}}</nowiki></span>\n");
add("html2wt", "<ref> in a <nowiki>", "Templating<nowiki><ref>{{simple template}}</ref></nowiki>\n\n\n");
add("html2wt", "<ref> in a <!--comment-->", "Templating\n\n\n");
add("html2wt", "<!--comment--> in a <ref> (bug 5384)", "Templating<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">Text</span>\n");
add("html2wt", "<references> after <gallery> (bug 6164)", "<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n\t\t* <div style=\"width: 155px\"> \t\t\t<div class=\"thumb\" style=\"width: 150px;\"><div style=\"margin:68px auto;\">http://example.com/images/thumb/3/3a/Foobar.jpg/120px-Foobar.jpg</div></div> \t\t\t<div class=\"gallerytext\">\t\t\t</div> \t\t</div>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">one</span>\n");
add("html2wt", "{{REVISIONID}} on page with <ref> (bug 6299)", "1337<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">elite</span>\n");
add("html2wt", "{{REVISIONID}} on page without <ref> (bug 6299 sanity check)", "1337\n");
add("html2wt", "Ref with content followed by blank ref", "<sup id=\"cite_ref-blank_1-0\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n<sup id=\"cite_ref-blank_1-1\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">↑ <sup>[[#cite_ref-blank_1-0|1.0]]</sup> <sup>[[#cite_ref-blank_1-1|1.1]]</sup></span> <span class=\"reference-text\">content</span>\n");
add("html2wt", "Blank ref followed by ref with content", "<sup id=\"cite_ref-blank_1-0\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n<sup id=\"cite_ref-blank_1-1\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">↑ <sup>[[#cite_ref-blank_1-0|1.0]]</sup> <sup>[[#cite_ref-blank_1-1|1.1]]</sup></span> <span class=\"reference-text\">content</span>\n");
add("html2wt", "Regression: non-blank ref \"0\" followed by ref with content", "<sup id=\"cite_ref-blank_1-0\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n<sup id=\"cite_ref-blank_1-1\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">↑ <sup>[[#cite_ref-blank_1-0|1.0]]</sup> <sup>[[#cite_ref-blank_1-1|1.1]]</sup></span> <span class=\"reference-text\">0 <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; name \"blank\" defined multiple times with different content</span></span>\n");
add("html2wt", "Regression sanity check: non-blank ref \"1\" followed by ref with content", "<sup id=\"cite_ref-blank_1-0\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n<sup id=\"cite_ref-blank_1-1\" class=\"reference\">[[#cite_note-blank-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">↑ <sup>[[#cite_ref-blank_1-0|1.0]]</sup> <sup>[[#cite_ref-blank_1-1|1.1]]</sup></span> <span class=\"reference-text\">1 <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; name \"blank\" defined multiple times with different content</span></span>\n");
add("html2wt", "Ref names containing a number", "<sup id=\"cite_ref-test123test_1-0\" class=\"reference\">[[#cite_note-test123test-1|[1]]]</sup>\n<sup id=\"cite_ref-123test_2-0\" class=\"reference\">[[#cite_note-123test-2|[2]]]</sup>\n<sup id=\"cite_ref-test123_3-0\" class=\"reference\">[[#cite_note-test123-3|[3]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-test123test_1-0|↑]]</span> <span class=\"reference-text\">One</span>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-123test_2-0|↑]]</span> <span class=\"reference-text\">Two</span>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-test123_3-0|↑]]</span> <span class=\"reference-text\">Three</span>\n");
add("html2wt", "Erroneous refs", "<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag;\nname cannot be a simple integer. Use a descriptive title</span>\n\n<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: The opening <code><nowiki><ref></nowiki></code> tag is malformed or has a bad name</span>\n\n<sup id=\"cite_ref-bar_2-0\" class=\"reference\">[[#cite_note-bar-2|[2]]]</sup>\n\n<sup id=\"cite_ref-blankwithnoreference_3-0\" class=\"reference\">[[#cite_note-blankwithnoreference-3|[3]]]</sup>\n\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid parameter in <code><nowiki><references></nowiki></code> tag</span>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">Also zero, but differently! (Normal ref)</span>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-bar_2-0|↑]]</span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; no text was provided for refs named <code>bar</code></span>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-blankwithnoreference_3-0|↑]]</span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; no text was provided for refs named <code>blankwithnoreference</code></span>\n");
add("html2wt", "Simple <ref>, with <references/> in group", "Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\nWikipedia rocks!<sup id=\"cite_ref-2\" class=\"reference\">[[#cite_note-2|[note 1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">Proceeds of Rockology, vol. XXI</span>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-2|↑]]</span> <span class=\"reference-text\">Proceeds of Rockology, vol. XXI</span>\n");
add("html2wt", "Simple <ref>, with <references/> in group, with groupname in Chinese", "AAA<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[参 1]]]</sup>BBB<sup id=\"cite_ref-2\" class=\"reference\">[[#cite_note-2|[注 1]]]</sup>CCC<sup id=\"cite_ref-3\" class=\"reference\">[[#cite_note-3|[参 2]]]</sup>\n\n; refs\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">ref a</span>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-3|↑]]</span> <span class=\"reference-text\">ref c</span>\n\n; notes\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-2|↑]]</span> <span class=\"reference-text\">note b</span>\n");
add("html2wt", "<ref> defined in <references>", "<sup id=\"cite_ref-foo_1-0\" class=\"reference\">[[#cite_note-foo-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-foo_1-0|↑]]</span> <span class=\"reference-text\">BAR</span>\n");
add("html2wt", "<ref> defined in <references> called with #tag", "<sup id=\"cite_ref-foo_1-0\" class=\"reference\">[[#cite_note-foo-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-foo_1-0|↑]]</span> <span class=\"reference-text\">BAR</span>\n");
add("html2wt", "<ref> defined in <references> error conditions", "<sup id=\"cite_ref-foo_1-0\" class=\"reference\">[[#cite_note-foo-1|[2 1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-foo_1-0|↑]]</span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; no text was provided for refs named <code>foo</code></span>\n\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: <code><nowiki><ref></nowiki></code> tag with name \"unused\" defined in <code><nowiki><references></nowiki></code> is not used in prior text.</span>\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: <code><nowiki><ref></nowiki></code> tag in <code><nowiki><references></nowiki></code> has conflicting group attribute \"1\".</span>\n<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: <code><nowiki><ref></nowiki></code> tag defined in <code><nowiki><references></nowiki></code> has no name attribute.</span>\n");
add("html2wt", "<ref> with custom group link with number names in Klingon", "Wikipedia rocks!<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[wa']]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">Proceeds of Rockology, vol. XXI</span>\n");
add("html2wt", "Bug 31374 regression check: nested strip items", "<sup id=\"cite_ref-2\" class=\"reference\">[[#cite_note-2|[Note 1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-2|↑]]</span> <span class=\"reference-text\">note<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup></span>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\">reference</span>\n");
add("html2wt", "Bug 13073 regression check: wrapped <references>", "<sup id=\"cite_ref-1\" class=\"reference\">[[#cite_note-1|[1]]]</sup>\n\n<div>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-1|↑]]</span> <span class=\"reference-text\"> foo</span>\n\n</div>\n");
add("html2wt", "<ref> with no name and no content.", "Bla.<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag;\nrefs with no name must have content</span>\n");
add("html2wt", "<ref> with an empty-string name parameter and no content.", "Bla.<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag;\nrefs with no name must have content</span>\n");
add("html2wt", "<ref> with a non-empty name parameter and no content.", "Bla.<sup id=\"cite_ref-void_1-0\" class=\"reference\">[[#cite_note-void-1|[1]]]</sup>\n# <span class=\"mw-cite-backlink\">[[#cite_ref-void_1-0|↑]]</span> <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; no text was provided for refs named <code>void</code></span>\n");
add("html2wt", "<ref>s with the follow parameter", "Page one.<sup id=\"cite_ref-beginning_1-0\" class=\"reference\">[[#cite_note-beginning-1|[1]]]</sup>\n\nPage two.\n\n== References ==\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-beginning_1-0|↑]]</span> <span class=\"reference-text\">First page footnote text. Second page footnote text.</span>\n");
add("html2wt", "<ref> with both name and follow parameters - invalid", "Page one.<span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag;\ninvalid names, e.g. too many</span>\n");
add("html2wt", "Multiple definition (outside <references/>)", "<sup id=\"cite_ref-a_1-0\" class=\"reference\">[[#cite_note-a-1|[1]]]</sup>\n<sup id=\"cite_ref-a_1-1\" class=\"reference\">[[#cite_note-a-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">↑ <sup>[[#cite_ref-a_1-0|1.0]]</sup> <sup>[[#cite_ref-a_1-1|1.1]]</sup></span> <span class=\"reference-text\">abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; name \"a\" defined multiple times with different content</span></span>\n");
add("html2wt", "Multiple definition (inside <references/>)", "<sup id=\"cite_ref-a_1-0\" class=\"reference\">[[#cite_note-a-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-a_1-0|↑]]</span> <span class=\"reference-text\">abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; name \"a\" defined multiple times with different content</span></span>\n");
add("html2wt", "Multiple definition (mixed outside/inside)", "<sup id=\"cite_ref-a_1-0\" class=\"reference\">[[#cite_note-a-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-a_1-0|↑]]</span> <span class=\"reference-text\">abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; name \"a\" defined multiple times with different content</span></span>\n");
add("html2wt", "Multiple definition (inside {{#tag:references}})", "<sup id=\"cite_ref-a_1-0\" class=\"reference\">[[#cite_note-a-1|[1]]]</sup>\n\n# <span class=\"mw-cite-backlink\">[[#cite_ref-a_1-0|↑]]</span> <span class=\"reference-text\">abc <span class=\"error mw-ext-cite-error\" lang=\"en\" dir=\"ltr\">Cite error: Invalid <code><nowiki><ref></nowiki></code> tag; name \"a\" defined multiple times with different content</span></span>\n");
add("html2wt", "Ref: 3. spaces in ref-names should be ignored", "A <ref name=\"x\">foo</ref>\nB <ref name=\"x\" />\nC <ref name=\"x\" />\n<references />");
add("html2wt", "Ref: 5. body should accept generic wikitext", "A <ref>This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("html2wt", "Ref: 6. indent-pres should not be output in ref-body", "A <ref>foo\n bar\n baz\n</ref>\n\n<references />");
add("html2wt", "Ref: 7. No p-wrapping in ref-body", "A <ref>foo\nbar\nbaz\nbooz\n</ref>\n\n<references />");
add("html2wt", "Ref: 8. transclusion wikitext has lower precedence", "A <ref><nowiki>foo {{echo|</nowiki></ref><nowiki> B C}}</nowiki>\n<references />");
add("html2wt", "Ref: 9. unclosed comments should not leak out of ref-body", "A <ref>foo <!----></ref> B C\n<references />");
add("html2wt", "Ref: 10. Unclosed HTML tags should not leak out of ref-body", "A <ref><b> foo </ref> B C\n\n<references />");
add("html2wt", "References: 9. Generate missing references list at the end", "A <ref>foo</ref> B <ref group=\"inexistent\">bar</ref>\n<references />\n<references group=\"inexistent\" />");


// Blacklist for selser
add("selser", "Simple <ref>, no <references/> [2,0,0]", "1618kiq\n\nWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>");
add("selser", "Simple <ref>, no <references/> [3,0,0]", "\n");
add("selser", "Simple <ref>, no <references/> [[3,0],0,0]", "<ref>Proceeds of Rockology, vol. XXI</ref>\n");
add("selser", "Simple <ref>, no <references/> [[4,0],2,0]", "vkomsc<ref>Proceeds of Rockology, vol. XXI</ref>\n\nx9946h\n");
add("selser", "Simple <ref>, no <references/> [4,4,0]", "1xru5nr\n\n17y1u9b");
add("selser", "Simple <ref>, no <references/> [4,0,0]", "11giwoh\n");
add("selser", "Simple <ref>, no <references/> [3,4,0]", "1um7ibt");
add("selser", "Simple <ref>, no <references/> [[2,0],0,0]", "10ki8pmWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n");
add("selser", "Simple <ref>, no <references/> [3,2,0]", "163orz7\n");
add("selser", "Simple <ref>, no <references/> [1,2,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n18t4cte\n");
add("selser", "Simple <ref>, no <references/> [[4,0],4,0]", "4q5ua9<ref>Proceeds of Rockology, vol. XXI</ref>\n\n1e0uxzv");
add("selser", "Simple <ref>, no <references/> [1,4,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\nbzza9r");
add("selser", "Simple <ref>, no <references/> [4,2,0]", "1s4gyw0\n\n1ixbzjf\n");
add("selser", "Simple <ref>, no <references/> [0,4,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n1rdol44");
add("selser", "Simple <ref>, no <references/> [[4,0],0,0]", "1l3ibd<ref>Proceeds of Rockology, vol. XXI</ref>\n");
add("selser", "Simple <ref>, no <references/> [0,3,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>");
add("selser", "Simple <ref>, no <references/> [[3,0],3,0]", "<ref>Proceeds of Rockology, vol. XXI</ref>");
add("selser", "Simple <ref>, no <references/> [1,0,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n");
add("selser", "Simple <ref>, no <references/> [0,2,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\nhlajmc\n");
add("selser", "Simple <ref>, with <references/> [0,3,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref><references/>");
add("selser", "Simple <ref>, with <references/> [[3,0],0,0]", "<ref>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [0,4,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\nac295a<references/>");
add("selser", "Simple <ref>, with <references/> [2,4,0]", "1pgtgnm\n\nWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\npkn61x<references/>");
add("selser", "Simple <ref>, with <references/> [1,0,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [3,0,0]", "\n<references/>");
add("selser", "Simple <ref>, with <references/> [[4,0],2,0]", "jgrqqr<ref>Proceeds of Rockology, vol. XXI</ref>\n\n6evs22\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [[4,0],4,0]", "sry0si<ref>Proceeds of Rockology, vol. XXI</ref>\n\n1lcnz50<references/>");
add("selser", "Simple <ref>, with <references/> [4,0,0]", "cq6zns\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [4,2,0]", "1moyztu\n\n5fvpzu\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [[2,0],2,0]", "ldo2d3Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n1k84dd\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [[2,0],0,0]", "bx6qyqWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [3,4,0]", "1u7einu<references/>");
add("selser", "Simple <ref>, with <references/> [4,3,0]", "yy0v55<references/>");
add("selser", "Simple <ref>, with <references/> [4,4,0]", "1i10sg1\n\n1pfvs3z<references/>");
add("selser", "Simple <ref>, with <references/> [[3,0],4,0]", "<ref>Proceeds of Rockology, vol. XXI</ref>\n\nykrjys<references/>");
add("selser", "Simple <ref>, with <references/> [[4,0],0,0]", "zhh349<ref>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [[3,0],2,0]", "<ref>Proceeds of Rockology, vol. XXI</ref>\n\nqftp90\n\n<references/>");
add("selser", "Simple <ref>, with <references/> [1,2,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\n\n2uaj73\n\n<references/>");
add("selser", "<ref> with a simple template [[4,0],0,0]", "lb3xqw<ref>{{simple template}}</ref>\n\n<references/>");
add("selser", "<ref> with a simple template [1,0,0]", "Templating<ref>{{simple template}}</ref>\n\n<references/>");
add("selser", "<ref> with a simple template [3,4,0]", "13kdddv<references/>");
add("selser", "<ref> with a simple template [3,0,0]", "\n<references/>");
add("selser", "<ref> with a simple template [4,0,0]", "1x2sbvn\n\n<references/>");
add("selser", "<ref> with a simple template [[2,0],0,0]", "8rkf5iTemplating<ref>{{simple template}}</ref>\n\n<references/>");
add("selser", "<ref> with a simple template [1,4,0]", "Templating<ref>{{simple template}}</ref>\n\n1arywk<references/>");
add("selser", "<ref> with a simple template [[2,0],4,0]", "1ylwnw5Templating<ref>{{simple template}}</ref>\n\n1dilcka<references/>");
add("selser", "<ref> with a simple template [[3,0],0,0]", "<ref>{{simple template}}</ref>\n\n<references/>");
add("selser", "<ref> with a simple template [2,0,0]", "lqtr8f\n\nTemplating<ref>{{simple template}}</ref>\n\n<references/>");
add("selser", "<ref> with a simple template [[4,0],4,0]", "lvn73v<ref>{{simple template}}</ref>\n\n13g5xj1<references/>");
add("selser", "<ref> with a simple template [2,4,0]", "lmnok8\n\nTemplating<ref>{{simple template}}</ref>\n\n11ciaoq<references/>");
add("selser", "<ref> with a simple template [[3,0],3,0]", "<ref>{{simple template}}</ref><references/>");
add("selser", "<ref> with a simple template [0,3,0]", "Templating<ref>{{simple template}}</ref><references/>");
add("selser", "<ref> with a simple template [0,2,0]", "Templating<ref>{{simple template}}</ref>\n\n1467oep\n\n<references/>");
add("selser", "<ref> with a simple template [4,3,0]", "11zr9xh<references/>");
add("selser", "<ref> with a simple template [4,4,0]", "1saychx\n\n1k2o19k<references/>");
add("selser", "<ref> with a simple template [1,2,0]", "Templating<ref>{{simple template}}</ref>\n\n6zmw0a\n\n<references/>");
add("selser", "<ref> with a simple template [3,2,0]", "16thjyc\n\n<references/>");
add("selser", "<ref> with a <nowiki> [3,0,0]", "\n<references/>");
add("selser", "<ref> with a <nowiki> [[2,0],4,0]", "815zw4Templating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n1vxfv0d<references/>");
add("selser", "<ref> with a <nowiki> [0,3,0]", "Templating<ref><nowiki>{{simple template}}</nowiki></ref><references/>");
add("selser", "<ref> with a <nowiki> [[4,0],2,0]", "sn49i7<ref><nowiki>{{simple template}}</nowiki></ref>\n\njfscwo\n\n<references/>");
add("selser", "<ref> with a <nowiki> [2,0,0]", "1slm8od\n\nTemplating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n<references/>");
add("selser", "<ref> with a <nowiki> [4,0,0]", "zlvr2w\n\n<references/>");
add("selser", "<ref> with a <nowiki> [[3,0],3,0]", "<ref><nowiki>{{simple template}}</nowiki></ref><references/>");
add("selser", "<ref> with a <nowiki> [[3,0],0,0]", "<ref><nowiki>{{simple template}}</nowiki></ref>\n\n<references/>");
add("selser", "<ref> with a <nowiki> [4,4,0]", "14zf7do\n\n1y7pom4<references/>");
add("selser", "<ref> with a <nowiki> [4,2,0]", "1jtfzh7\n\n1xaufno\n\n<references/>");
add("selser", "<ref> with a <nowiki> [0,4,0]", "Templating<ref><nowiki>{{simple template}}</nowiki></ref>\n\nwwyku0<references/>");
add("selser", "<ref> with a <nowiki> [1,0,0]", "Templating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n<references/>");
add("selser", "<ref> with a <nowiki> [1,3,0]", "Templating<ref><nowiki>{{simple template}}</nowiki></ref><references/>");
add("selser", "<ref> with a <nowiki> [[2,0],3,0]", "kskp27Templating<ref><nowiki>{{simple template}}</nowiki></ref><references/>");
add("selser", "<ref> with a <nowiki> [[2,0],2,0]", "x0plwbTemplating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n1pwd9nz\n\n<references/>");
add("selser", "<ref> with a <nowiki> [4,3,0]", "18tlh49<references/>");
add("selser", "<ref> with a <nowiki> [1,4,0]", "Templating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n3vxy3g<references/>");
add("selser", "<ref> with a <nowiki> [2,2,0]", "bihj86\n\nTemplating<ref><nowiki>{{simple template}}</nowiki></ref>\n\n12p56t9\n\n<references/>");
add("selser", "<ref> with a <nowiki> [3,3,0]", "<references/>");
add("selser", "<ref> in a <nowiki> [[4,0],0,0]", "bxdyes<nowiki><ref>{{simple template}}</ref></nowiki>\n\n<references/>");
add("selser", "<ref> in a <nowiki> [4,0,0]", "136ce6g\n\n<references/>");
add("selser", "<ref> in a <nowiki> [[0,[4]],3,0]", "Templating<nowiki>5q0mfn</nowiki><references/>");
add("selser", "<ref> in a <nowiki> [4,2,0]", "11g8nx5\n\n1nnng1i\n\n<references/>");
add("selser", "<ref> in a <nowiki> [1,3,0]", "Templating<nowiki><ref>{{simple template}}</ref></nowiki><references/>");
add("selser", "<ref> in a <nowiki> [[0,1],2,0]", "Templating<nowiki><ref>{{simple template}}</ref></nowiki>\n\n1b84t9k\n\n<references/>");
add("selser", "<ref> in a <nowiki> [[0,[4]],2,0]", "Templating<nowiki>hq50k0</nowiki>\n\n2l8xhs\n\n<references/>");
add("selser", "<ref> in a <nowiki> [0,4,0]", "Templating<nowiki><ref>{{simple template}}</ref></nowiki>\n\nq3ojhu<references/>");
add("selser", "<ref> in a <nowiki> [4,3,0]", "1j940eo<references/>");
add("selser", "<ref> in a <nowiki> [[0,[2]],4,0]", "Templating<nowiki>jha8g5<ref>{{simple template}}</ref></nowiki>\n\n5i0zgj<references/>");
add("selser", "<ref> in a <nowiki> [3,4,0]", "1bhwg82<references/>");
add("selser", "<ref> in a <nowiki> [4,4,0]", "1e7ynwk\n\n1ob6y4b<references/>");
add("selser", "<ref> in a <nowiki> [[4,[4]],0,0]", "1kka8jp<nowiki>1l2n4rh</nowiki>\n\n<references/>");
add("selser", "<ref> in a <nowiki> [2,2,0]", "1cf0bfy\n\nTemplating<nowiki><ref>{{simple template}}</ref></nowiki>\n\nu7tl59\n\n<references/>");
add("selser", "<ref> in a <nowiki> [[0,2],4,0]", "Templatingzgch3g<nowiki><ref>{{simple template}}</ref></nowiki>\n\niasye4<references/>");
add("selser", "<ref> in a <nowiki> [[3,3],0,0]", "\n<references/>");
add("selser", "<ref> in a <nowiki> [1,2,0]", "Templating<nowiki><ref>{{simple template}}</ref></nowiki>\n\n2qfc6m\n\n<references/>");
add("selser", "<ref> in a <nowiki> [3,0,0]", "\n<references/>");
add("selser", "<ref> in a <nowiki> [[3,1],2,0]", "<nowiki><ref>{{simple template}}</ref></nowiki>\n\n19ujeno\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [1,0,3,0]", "Templating<!--<ref>{{simple template}}</ref>--><references/>");
add("selser", "<ref> in a <!--comment--> [3,2,2,0]", "12jhvgu<!--<ref>{{simple template}}</ref>-->\n\n18g17b7\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [1,4,0,0]", "Templating\n\n5zp45m\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [4,0,0,0]", "1glzb0q<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [2,2,2,0]", "zmfmbh\n\nTemplating\n\n1tj3fp8<!--<ref>{{simple template}}</ref>-->\n\nezc5i\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [2,0,0,0]", "1tl5fpe\n\nTemplating<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [[2],0,0,0]", "spqmvkTemplating<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [1,2,0,0]", "Templating\n\n1bmgt0<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [[3],0,3,0]", "<!--<ref>{{simple template}}</ref>--><references/>");
add("selser", "<ref> in a <!--comment--> [[3],4,3,0]", "\nzvdyi6<references/>");
add("selser", "<ref> in a <!--comment--> [0,0,4,0]", "Templating<!--<ref>{{simple template}}</ref>-->\n\nv5f64a<references/>");
add("selser", "<ref> in a <!--comment--> [0,2,0,0]", "Templating\n\njl0u6z<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [4,2,0,0]", "cxbie7\n\n193651b<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [[4],4,0,0]", "rwr21j\n\n3i4vbo\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [0,4,4,0]", "Templating\n\njorx4s\n\novhwlu<references/>");
add("selser", "<ref> in a <!--comment--> [[4],0,0,0]", "55xsbo<!--<ref>{{simple template}}</ref>-->\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [2,0,2,0]", "1o6rlh6\n\nTemplating<!--<ref>{{simple template}}</ref>-->\n\nkvy2h5\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [2,3,0,0]", "1jfh1jf\n\nTemplating\n\n<references/>");
add("selser", "<ref> in a <!--comment--> [0,3,0,0]", "Templating\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[3,0],0,0]", "<ref>Text<!--comment--></ref>\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [1,0,0]", "Templating<ref>Text<!--comment--></ref>\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [0,4,0]", "Templating<ref>Text<!--comment--></ref>\n\n1eah8c<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [4,0,0]", "1j2vvn2\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[2,0],0,0]", "qhx68cTemplating<ref>Text<!--comment--></ref>\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [1,4,0]", "Templating<ref>Text<!--comment--></ref>\n\ng8kaa8<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [2,4,0]", "45xvae\n\nTemplating<ref>Text<!--comment--></ref>\n\nz6f8s6<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[2,0],3,0]", "kd52goTemplating<ref>Text<!--comment--></ref><references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [0,3,0]", "Templating<ref>Text<!--comment--></ref><references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [0,2,0]", "Templating<ref>Text<!--comment--></ref>\n\n5uxjrm\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [2,3,0]", "199xj06\n\nTemplating<ref>Text<!--comment--></ref><references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [4,2,0]", "1yeg3bf\n\n1oxoafc\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[4,0],0,0]", "ipbopo<ref>Text<!--comment--></ref>\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [2,0,0]", "8bxrbo\n\nTemplating<ref>Text<!--comment--></ref>\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[4,0],4,0]", "h89q3f<ref>Text<!--comment--></ref>\n\n11enah3<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[4,0],2,0]", "qwc1z0<ref>Text<!--comment--></ref>\n\n1kyl4mw\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[3,0],2,0]", "<ref>Text<!--comment--></ref>\n\n1rgnu6\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [2,2,0]", "q3ljrl\n\nTemplating<ref>Text<!--comment--></ref>\n\nieou6m\n\n<references/>");
add("selser", "<!--comment--> in a <ref> (bug 5384) [[2,0],2,0]", "20seleTemplating<ref>Text<!--comment--></ref>\n\n1e060aj\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,2,0,0,0]", "<ref>one</ref>\n\nojcdyo\n\n<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,4,0,2,0]", "<ref>one</ref>\n\n5bjxgx<gallery>Image:Foobar.jpg</gallery>r5xxp1\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [3,3,0,0,0]", "<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [3,0,0,3,0]", "\n<gallery>Image:Foobar.jpg</gallery><references/>");
add("selser", "<references> after <gallery> (bug 6164) [2,0,0,0,0]", "1t7eryg\n\n<ref>one</ref>\n\n<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,4,0,3,0]", "<ref>one</ref>\n\nur2cbg<gallery>Image:Foobar.jpg</gallery><references/>");
add("selser", "<references> after <gallery> (bug 6164) [3,0,0,2,0]", "\n<gallery>Image:Foobar.jpg</gallery>1cz0im3\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [2,3,0,0,0]", "1ofgkto\n\n<ref>one</ref><gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [1,0,0,0,0]", "<ref>one</ref>\n\n<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,0,0,4,0]", "<ref>one</ref>\n\n<gallery>Image:Foobar.jpg</gallery>rbafmv<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,3,0,4,0]", "<ref>one</ref><gallery>Image:Foobar.jpg</gallery>pa7vx<references/>");
add("selser", "<references> after <gallery> (bug 6164) [4,3,0,0,0]", "145fhvu<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,4,0,4,0]", "<ref>one</ref>\n\nxr7goe<gallery>Image:Foobar.jpg</gallery>1gt7p5z<references/>");
add("selser", "<references> after <gallery> (bug 6164) [4,4,0,2,0]", "xw5zvu\n\ned4faj<gallery>Image:Foobar.jpg</gallery>13rjxop\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [1,2,0,0,0]", "<ref>one</ref>\n\n5b6n8b\n\n<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [3,0,0,0,0]", "\n<gallery>Image:Foobar.jpg</gallery>\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [0,3,0,2,0]", "<ref>one</ref><gallery>Image:Foobar.jpg</gallery>bdtml6\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [2,0,0,2,0]", "1auvsbr\n\n<ref>one</ref>\n\n<gallery>Image:Foobar.jpg</gallery>171rlvk\n\n<references/>");
add("selser", "<references> after <gallery> (bug 6164) [4,2,0,4,0]", "1eftrz2\n\nwaul9f\n\n<gallery>Image:Foobar.jpg</gallery>yuawyo<references/>");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [0,3,0]", "{{REVISIONID}}<ref>elite</ref>");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [1,2,0]", "{{REVISIONID}}<ref>elite</ref>\n\n2w1ylf\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [2,2,0]", "1sqvfrd\n\n{{REVISIONID}}<ref>elite</ref>\n\nmuq1xa\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [3,0,0]", "\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [2,0,0]", "1sc27yk\n\n{{REVISIONID}}<ref>elite</ref>");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [0,4,0]", "{{REVISIONID}}<ref>elite</ref>\n\n6820q5");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [4,3,0]", "o1rvrs");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [1,4,0]", "{{REVISIONID}}<ref>elite</ref>\n\n755jmj");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [4,0,0]", "12u7cbe\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [1,3,0]", "{{REVISIONID}}<ref>elite</ref>");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [3,3,0]", "");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [0,2,0]", "{{REVISIONID}}<ref>elite</ref>\n\n42lwa3\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [3,4,0]", "hc2c9y");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [2,4,0]", "4mfowb\n\n{{REVISIONID}}<ref>elite</ref>\n\nq8t2f8");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [4,4,0]", "ibohng\n\n166o2nr");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [1,0,0]", "{{REVISIONID}}<ref>elite</ref>\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [2,3,0]", "dwrha5\n\n{{REVISIONID}}<ref>elite</ref>");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [3,2,0]", "kl65oh\n");
add("selser", "{{REVISIONID}} on page with <ref> (bug 6299) [4,2,0]", "5uvqjo\n\n7gww77\n");
add("selser", "Ref with content followed by blank ref [1,3,0,0,0]", "<ref name=\"blank\">content</ref>\n\n<ref name=\"blank\"/>\n\n<references/>");
add("selser", "Ref with content followed by blank ref [3,0,0,3,0]", "\n<ref name=\"blank\"/><references/>");
add("selser", "Ref with content followed by blank ref [0,3,4,4,0]", "<ref name=\"blank\">content</ref>\n\nkq1u2m\n\n1vq0g7y<references/>");
add("selser", "Ref with content followed by blank ref [4,0,1,3,0]", "1i0rav0\n\n<ref name=\"blank\"/><references/>");
add("selser", "Ref with content followed by blank ref [0,4,0,3,0]", "<ref name=\"blank\">content</ref>\n\nshsg9u\n\n<ref name=\"blank\"/><references/>");
add("selser", "Ref with content followed by blank ref [0,3,1,0,0]", "<ref name=\"blank\">content</ref>\n\n<ref name=\"blank\"/>\n\n<references/>");
add("selser", "Ref with content followed by blank ref [4,0,3,0,0]", "177jxhv\n\n<references/>");
add("selser", "Ref with content followed by blank ref [0,0,0,4,0]", "<ref name=\"blank\">content</ref>\n\n<ref name=\"blank\"/>\n\nunii5u<references/>");
add("selser", "Ref with content followed by blank ref [3,0,0,4,0]", "\n<ref name=\"blank\"/>\n\n1kspqll<references/>");
add("selser", "Ref with content followed by blank ref [1,3,1,2,0]", "<ref name=\"blank\">content</ref>\n\n<ref name=\"blank\"/>\n\nwfab6u\n\n<references/>");
add("selser", "Ref with content followed by blank ref [3,0,2,4,0]", "\n140yw3d\n\n<ref name=\"blank\"/>\n\n19b1euz<references/>");
add("selser", "Ref with content followed by blank ref [0,0,2,0,0]", "<ref name=\"blank\">content</ref>\n\ncycfe2\n\n<ref name=\"blank\"/>\n\n<references/>");
add("selser", "Ref with content followed by blank ref [0,0,1,3,0]", "<ref name=\"blank\">content</ref>\n\n<ref name=\"blank\"/><references/>");
add("selser", "Ref with content followed by blank ref [4,4,0,2,0]", "1eou0ss\n\n1hrex9q\n\n<ref name=\"blank\"/>\n\n1gx9civ\n\n<references/>");
add("selser", "Ref with content followed by blank ref [4,0,0,0,0]", "102jqv0\n\n<ref name=\"blank\"/>\n\n<references/>");
add("selser", "Ref with content followed by blank ref [0,4,2,0,0]", "<ref name=\"blank\">content</ref>\n\ntiwdz5\n\n1ghy8ad\n\n<ref name=\"blank\"/>\n\n<references/>");
add("selser", "Ref with content followed by blank ref [0,0,3,0,0]", "<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Ref with content followed by blank ref [1,2,3,2,0]", "<ref name=\"blank\">content</ref>\n\nbovja2\n\n1l3kwy7\n\n<references/>");
add("selser", "Ref with content followed by blank ref [4,2,0,3,0]", "19i8cn8\n\n1i7mkad\n\n<ref name=\"blank\"/><references/>");
add("selser", "Ref with content followed by blank ref [4,0,0,4,0]", "1wgafau\n\n<ref name=\"blank\"/>\n\n1jfcrdq<references/>");
add("selser", "Blank ref followed by ref with content [0,2,0,0,0]", "<ref name=\"blank\"/>\n\nhegfnt\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Blank ref followed by ref with content [0,0,4,0,0]", "<ref name=\"blank\"/>\n\nr3u74s\n\n<references/>");
add("selser", "Blank ref followed by ref with content [1,0,0,0,0]", "<ref name=\"blank\"/>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Blank ref followed by ref with content [2,0,0,0,0]", "1k4tg3v\n\n<ref name=\"blank\"/>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Blank ref followed by ref with content [0,0,0,4,0]", "<ref name=\"blank\"/>\n\n<ref name=\"blank\">content</ref>\n\nvqa4r8<references/>");
add("selser", "Blank ref followed by ref with content [0,4,1,3,0]", "<ref name=\"blank\"/>\n\n2oyky3\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Blank ref followed by ref with content [3,2,4,3,0]", "1qkxkfu\n\n1f7e3wf<references/>");
add("selser", "Blank ref followed by ref with content [2,2,3,0,0]", "1vrai6u\n\n<ref name=\"blank\"/>\n\nl4hz2e\n\n<references/>");
add("selser", "Blank ref followed by ref with content [3,0,0,2,0]", "\n<ref name=\"blank\">content</ref>\n\n13vx502\n\n<references/>");
add("selser", "Blank ref followed by ref with content [0,4,0,3,0]", "<ref name=\"blank\"/>\n\n6o33ca\n\n<ref name=\"blank\">content</ref><references/>");
add("selser", "Blank ref followed by ref with content [3,2,1,3,0]", "1q469t2\n\n<ref name=\"blank\">content</ref><references/>");
add("selser", "Blank ref followed by ref with content [4,0,2,0,0]", "173370a\n\n1j87mwj\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Blank ref followed by ref with content [0,4,0,0,0]", "<ref name=\"blank\"/>\n\n1wqpem3\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Blank ref followed by ref with content [3,4,3,0,0]", "1512cr6\n\n<references/>");
add("selser", "Blank ref followed by ref with content [2,4,0,2,0]", "1smy595\n\n<ref name=\"blank\"/>\n\nw67z4d\n\n<ref name=\"blank\">content</ref>\n\nzsl98a\n\n<references/>");
add("selser", "Blank ref followed by ref with content [0,4,2,2,0]", "<ref name=\"blank\"/>\n\ny1m954\n\n103gv13\n\n<ref name=\"blank\">content</ref>\n\n1uslacl\n\n<references/>");
add("selser", "Blank ref followed by ref with content [0,0,4,4,0]", "<ref name=\"blank\"/>\n\nmb9mxt\n\nejkd51<references/>");
add("selser", "Blank ref followed by ref with content [1,0,4,4,0]", "<ref name=\"blank\"/>\n\n16wgg3z\n\nsz3xdu<references/>");
add("selser", "Blank ref followed by ref with content [2,3,0,0,0]", "dmkd7e\n\n<ref name=\"blank\"/>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [4,3,0,3,0]", "1i7daij\n\n<ref name=\"blank\">content</ref><references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,0,0,4,0]", "<ref name=\"blank\">0</ref>\n\n<ref name=\"blank\">content</ref>\n\nis44vx<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [4,4,0,2,0]", "1is855h\n\n1q13jpf\n\n<ref name=\"blank\">content</ref>\n\n15f5ydm\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [1,4,0,4,0]", "<ref name=\"blank\">0</ref>\n\n93cdjt\n\n<ref name=\"blank\">content</ref>\n\n1ngflci<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,0,1,0,0]", "<ref name=\"blank\">0</ref>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,3,3,2,0]", "<ref name=\"blank\">0</ref>\n\nwmi5ym\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [3,2,3,3,0]", "1nn3f3p\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [3,2,0,3,0]", "1lbfpn8\n\n<ref name=\"blank\">content</ref><references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [3,2,0,0,0]", "1tit865\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [2,4,4,0,0]", "17i4led\n\n<ref name=\"blank\">0</ref>\n\nv91sd2\n\n1opjqri\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [4,2,0,0,0]", "1ahzk52\n\n1xjd0vp\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,0,2,0,0]", "<ref name=\"blank\">0</ref>\n\npamfg2\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,0,3,2,0]", "<ref name=\"blank\">0</ref>\n\nq39hs3\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [4,2,0,4,0]", "1su8zsj\n\n1l2e2ni\n\n<ref name=\"blank\">content</ref>\n\n1n8dq9i<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,2,1,0,0]", "<ref name=\"blank\">0</ref>\n\nhqrxsy\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,3,0,3,0]", "<ref name=\"blank\">0</ref>\n\n<ref name=\"blank\">content</ref><references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [0,2,3,2,0]", "<ref name=\"blank\">0</ref>\n\n3qi0ie\n\n1f8ysu1\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [2,2,2,2,0]", "1muuvq7\n\n<ref name=\"blank\">0</ref>\n\nxlzkt5\n\n1jx5sqb\n\n<ref name=\"blank\">content</ref>\n\n8holmt\n\n<references/>");
add("selser", "Regression: non-blank ref \"0\" followed by ref with content [2,0,1,0,0]", "1k13q9s\n\n<ref name=\"blank\">0</ref>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,0,3,0,0]", "<ref name=\"blank\">1</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [1,0,4,2,0]", "<ref name=\"blank\">1</ref>\n\n5ar3d3\n\nl7vvs6\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [1,2,0,0,0]", "<ref name=\"blank\">1</ref>\n\n7ypg7n\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,0,0,4,0]", "<ref name=\"blank\">1</ref>\n\n<ref name=\"blank\">content</ref>\n\n6pvwho<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [3,4,3,0,0]", "1dtufe6\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [2,3,3,0,0]", "1xi5esd\n\n<ref name=\"blank\">1</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [2,0,3,0,0]", "1xglj0x\n\n<ref name=\"blank\">1</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [1,4,1,2,0]", "<ref name=\"blank\">1</ref>\n\n3sk0ge\n\n<ref name=\"blank\">content</ref>\n\n1s1e62u\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [4,0,0,4,0]", "17chaeh\n\n<ref name=\"blank\">content</ref>\n\n1hw7bsk<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,4,0,4,0]", "<ref name=\"blank\">1</ref>\n\nijssqv\n\n<ref name=\"blank\">content</ref>\n\n1gt2n9a<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,0,4,0,0]", "<ref name=\"blank\">1</ref>\n\n7u3nkm\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [4,0,3,2,0]", "1tpmtwv\n\n1oz5iy3\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,4,3,0,0]", "<ref name=\"blank\">1</ref>\n\n1s16l5\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [1,3,1,0,0]", "<ref name=\"blank\">1</ref>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [2,3,3,2,0]", "zm04t0\n\n<ref name=\"blank\">1</ref>\n\nk5qbiv\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [1,2,4,0,0]", "<ref name=\"blank\">1</ref>\n\ncsktpa\n\nzux7se\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [3,0,3,4,0]", "\n11rqfk3<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [1,0,0,0,0]", "<ref name=\"blank\">1</ref>\n\n<ref name=\"blank\">content</ref>\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,4,4,2,0]", "<ref name=\"blank\">1</ref>\n\nj0jln\n\n1hjeriq\n\n1ghfa4n\n\n<references/>");
add("selser", "Regression sanity check: non-blank ref \"1\" followed by ref with content [0,0,0,2,0]", "<ref name=\"blank\">1</ref>\n\n<ref name=\"blank\">content</ref>\n\nowjukk\n\n<references/>");
add("selser", "Erroneous refs [0,0,4,2,4,0,0,3,0,0,0,0,0]", "<ref name=\"0\">Zero</ref>\n\n5fiuuy\n\n1r3vk7r\n\nye789s\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [3,2,0,0,1,2,0,0,1,0,0,2,0]", "1phopva\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\n167t8v8\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />e8juxg\n\n<references />");
add("selser", "Erroneous refs [0,3,0,0,0,2,0,0,1,0,0,3,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\nvxfzfv\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" /><references />");
add("selser", "Erroneous refs [2,0,4,3,4,4,0,0,0,3,0,0,0]", "1ehl52g\n\n<ref name=\"0\">Zero</ref>\n\npetz7r\n\nyvg893\n\n89bcw4\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" /><references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [0,0,0,2,0,0,4,2,0,0,0,0,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\nu7n0zu\n\n<ref />\n\n70aqbt\n\n9vwwfa\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [0,4,0,3,0,0,0,0,0,3,0,0,0]", "<ref name=\"0\">Zero</ref>\n\nec3dpf\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" /><references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [3,0,0,3,4,0,4,3,2,4,0,0,0]", "\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n1ipulhc\n\n100gk67\n\ni2dlju\n\n<ref name=\"blankwithnoreference\" />\n\niqq6bi<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [3,0,0,0,0,4,0,0,3,0,0,2,0]", "\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\n1y2hyjq\n\n<ref name=\"foo\" name=\"bar\" />\n\n<references name=\"quasit\" />1co7pmk\n\n<references />");
add("selser", "Erroneous refs [3,2,4,2,4,0,0,2,3,0,0,4,0]", "1nkyb74\n\n14bmiw4\n\n1qiocud\n\nh9kl7a\n\n<ref name=\"foo\" name=\"bar\" />\n\n1r6og91\n\n<references name=\"quasit\" />1jy8o0b<references />");
add("selser", "Erroneous refs [3,0,4,2,0,2,0,0,0,0,0,4,0]", "\n1c5u2u1\n\n11wpc6k\n\n<ref />\n\niseoxu\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />1komb4y<references />");
add("selser", "Erroneous refs [0,0,1,0,0,4,0,3,1,2,0,4,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\nmwzizm\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n1dp2vom\n\n<references name=\"quasit\" />1g0m87q<references />");
add("selser", "Erroneous refs [1,0,0,3,1,3,1,0,2,0,0,4,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\n<ref name=\"foo\" name=\"bar\" />\n\n16q7f29\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />onqj6h<references />");
add("selser", "Erroneous refs [2,0,3,2,0,0,2,3,4,0,0,0,0]", "151n3wc\n\n<ref name=\"0\">Zero</ref>\n\nsyitr4\n\n<ref />\n\n3ge2b1\n\n<ref name=\"foo\" name=\"bar\" />\n\nocdgrg\n\n<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [2,0,2,2,0,4,2,0,0,3,0,3,0]", "1aa5hxv\n\n<ref name=\"0\">Zero</ref>\n\njuhnew\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\np140s5\n\n<ref />\n\nqfta45\n\n1v3f3j9\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" /><references name=\"quasit\" /><references />");
add("selser", "Erroneous refs [4,4,4,0,0,0,0,3,1,2,0,0,0]", "1cfen4e\n\n1ncvdsc\n\n11w4mcc\n\n<ref />\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n1w92yl4\n\n<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [0,0,2,0,2,2,2,4,0,0,0,2,0]", "<ref name=\"0\">Zero</ref>\n\nwe26ed\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n1yu98fg\n\n<ref />\n\na84x3d\n\ns774dh\n\n<ref name=\"foo\" name=\"bar\" />\n\n1rpidtp\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />raefy5\n\n<references />");
add("selser", "Erroneous refs [0,0,1,0,0,0,0,0,0,4,0,0,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\nam6bb8<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [3,0,0,2,0,3,3,0,2,4,0,0,0]", "\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n1iau755\n\n<ref />\n\n1awciva\n\n<ref name=\"blankwithnoreference\" />\n\noqdrva<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [1,3,0,0,4,4,1,2,0,0,0,0,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\ndorkkr\n\n1c5f38j\n\n<ref name=\"bar\" />\n\ngw78hp\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />\n\n<references />");
add("selser", "Erroneous refs [0,0,0,3,0,2,2,3,1,0,0,2,0]", "<ref name=\"0\">Zero</ref>\n\n<ref>Also zero, but differently! (Normal ref)</ref>\n\n<ref />\n\nj7066e\n\n1a4dt8o\n\n<ref name=\"foo\" name=\"bar\" />\n\n<ref name=\"blankwithnoreference\" />\n\n<references name=\"quasit\" />10jb3z9\n\n<references />");
add("selser", "Simple <ref>, with <references/> in group [2,4,0,3,0]", "1lplvj4\n\nWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\njvb1n4<references/><references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [1,3,0,0,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref><references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [3,3,0,0,0]", "<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[2,0,0,0],0,0,0,0]", "pmtlzsWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [4,0,0,0,0]", "17bbke1\n\n<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[4,0,0,0],2,0,2,0]", "4x539<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n1mkvpob\n\n<references/>1tsfwrx\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [2,0,0,3,0]", "12ldvh1\n\nWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/><references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[2,0,2,0],2,0,0,0]", "v7rezuWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>aikd07\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n10cozbr\n\n<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [0,0,0,4,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>uz68nm<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [4,3,0,2,0]", "19hh0pn<references/>unhqcb\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [4,0,0,2,0]", "1uce4y9\n\n<references/>1l0u4bh\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[3,0,0,0],2,0,0,0]", "<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\npt5oay\n\n<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [4,0,0,3,0]", "1etled4\n\n<references/><references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[2,0,0,0],2,0,0,0]", "eb7u01Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\nslvigf\n\n<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[4,0,0,0],4,0,0,0]", "rkabia<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\nmrdr3e<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [0,0,0,2,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>v9q3he\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[0,0,4,0],0,0,3,0]", "Wikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>95iiri<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/><references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [[2,0,3,0],0,0,0,0]", "24b7ioWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref><ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>\n<references group=note/>");
add("selser", "Simple <ref>, with <references/> in group [2,0,0,0,0]", "1oz1aeu\n\nWikipedia rocks!<ref>Proceeds of Rockology, vol. XXI</ref>\nWikipedia rocks!<ref group=note>Proceeds of Rockology, vol. XXI</ref>\n\n<references/>\n<references group=note/>");
add("selser", "<ref> defined in <references> [0,2,0]", "<ref name=\"foo\"/>\n\nhxy5y3\n\n<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [0,3,0]", "<ref name=\"foo\"/><references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [2,4,0]", "1pixeop\n\n<ref name=\"foo\"/>\n\n1m7odyr<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [0,4,0]", "<ref name=\"foo\"/>\n\n8blj32<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [1,0,0]", "<ref name=\"foo\"/>\n\n<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [1,3,0]", "<ref name=\"foo\"/><references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [2,2,0]", "16h3504\n\n<ref name=\"foo\"/>\n\npnv0m8\n\n<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [2,0,0]", "14x1duy\n\n<ref name=\"foo\"/>\n\n<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [1,2,0]", "<ref name=\"foo\"/>\n\nklrjmz\n\n<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [1,4,0]", "<ref name=\"foo\"/>\n\n15v6d7g<references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> [2,3,0]", "1yh61uq\n\n<ref name=\"foo\"/><references>\n<ref name=\"foo\">BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> called with #tag [0,0,0,4,0]", "<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}\n\nbrccs7");
add("selser", "<ref> defined in <references> called with #tag [1,3,0,3,0]", "<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [0,4,3,0,0]", "<ref name=\"foo\"/>\n\namo6ws\n");
add("selser", "<ref> defined in <references> called with #tag [0,3,4,0,0]", "<ref name=\"foo\"/>\n\ny49ita\n");
add("selser", "<ref> defined in <references> called with #tag [3,4,0,0,0]", "1m7d9re\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [1,0,4,0,0]", "<ref name=\"foo\"/>\n\n7tn39e\n");
add("selser", "<ref> defined in <references> called with #tag [2,0,1,4,0]", "13owd1e\n\n<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}\n\nnrnshy");
add("selser", "<ref> defined in <references> called with #tag [0,4,0,0,0]", "<ref name=\"foo\"/>\n\nrdn99l\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [4,2,0,0,0]", "1clskzx\n\n1y73y10\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [4,0,3,3,0]", "g1897y\n\n");
add("selser", "<ref> defined in <references> called with #tag [4,2,3,4,0]", "12y1e85\n\n1kob1jt\n\n1thxs26");
add("selser", "<ref> defined in <references> called with #tag [0,3,3,3,0]", "<ref name=\"foo\"/>");
add("selser", "<ref> defined in <references> called with #tag [3,2,0,0,0]", "1rkze1y\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [0,3,0,3,0]", "<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [4,0,1,4,0]", "1g2r0d1\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}\n\nrdx18w");
add("selser", "<ref> defined in <references> called with #tag [0,0,0,3,0]", "<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}");
add("selser", "<ref> defined in <references> called with #tag [0,0,0,2,0]", "<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}\n\nfj0gen\n");
add("selser", "<ref> defined in <references> called with #tag [2,0,1,0,0]", "1d7653n\n\n<ref name=\"foo\"/>\n\n{{#tag:references|\n<ref name=\"foo\">BAR</ref>\n}}\n");
add("selser", "<ref> defined in <references> called with #tag [2,3,3,0,0]", "14bboaw\n\n<ref name=\"foo\"/>\n");
add("selser", "<ref> defined in <references> error conditions [0,0,0,4,0]", "<ref name=\"foo\" group=\"2\"/>\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>dxumdc");
add("selser", "<ref> defined in <references> error conditions [2,4,0,2,0]", "1pumkgv\n\n<ref name=\"foo\" group=\"2\"/>\n\njvx805<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>11l4u9n\n");
add("selser", "<ref> defined in <references> error conditions [1,0,0,3,0]", "<ref name=\"foo\" group=\"2\"/>\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [2,3,0,3,0]", "1n05ikg\n\n<ref name=\"foo\" group=\"2\"/><references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [0,0,0,2,0]", "<ref name=\"foo\" group=\"2\"/>\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>lmy9sp\n");
add("selser", "<ref> defined in <references> error conditions [1,4,0,2,0]", "<ref name=\"foo\" group=\"2\"/>\n\n51uldm<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>1ax3od5\n");
add("selser", "<ref> defined in <references> error conditions [2,0,0,2,0]", "16xtqf7\n\n<ref name=\"foo\" group=\"2\"/>\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>l8coyc\n");
add("selser", "<ref> defined in <references> error conditions [0,3,0,3,0]", "<ref name=\"foo\" group=\"2\"/><references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [3,2,0,0,0]", "13kk1ep\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [2,3,0,0,0]", "s21hov\n\n<ref name=\"foo\" group=\"2\"/><references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [0,0,0,3,0]", "<ref name=\"foo\" group=\"2\"/>\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [4,4,0,0,0]", "1pq6xj8\n\n1s9st80<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [4,0,0,0,0]", "1z01g6o\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [0,4,0,3,0]", "<ref name=\"foo\" group=\"2\"/>\n\ne6zpzy<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [0,3,0,0,0]", "<ref name=\"foo\" group=\"2\"/><references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [1,3,0,3,0]", "<ref name=\"foo\" group=\"2\"/><references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [0,2,0,0,0]", "<ref name=\"foo\" group=\"2\"/>\n\nxnimd\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>");
add("selser", "<ref> defined in <references> error conditions [3,2,0,2,0]", "15osac8\n\n<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>17d9429\n");
add("selser", "<ref> defined in <references> error conditions [0,4,0,4,0]", "<ref name=\"foo\" group=\"2\"/>\n\nyooxfw<references group=\"2\">\n<ref name=\"foo\"/>\n<ref name=\"unused\">BAR</ref>\n<ref name=\"foo\" group=\"1\">bad group</ref>\n<ref>BAR BAR</ref>\n</references>cfercb");
add("selser", "<ref> with custom group link with number names in Klingon [4,2,0]", "1dw6psd\n\n1rj3pfy\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [0,2,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\nda0qzb\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [1,0,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [0,4,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\nbbq4ik<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [2,0,0]", "1bajwfe\n\nWikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [1,2,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\nha565t\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [[2,0],4,0]", "u6bhegWikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n1ww2iyh<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [[4,0],0,0]", "ke30tl<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [3,2,0]", "1hl07p8\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [[3,0],0,0]", "<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [3,0,0]", "\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [3,4,0]", "12itx1o<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [[4,0],3,0]", "rzesd6<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref><references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [4,0,0]", "2ysb9o\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [4,3,0]", "1hktzyz<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [[2,0],0,0]", "r0sr2bWikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [1,4,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref>\n\n1yfv8ni<references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [0,3,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref><references group=\"klingon\"/>");
add("selser", "<ref> with custom group link with number names in Klingon [1,3,0]", "Wikipedia rocks!<ref group=\"klingon\">Proceeds of Rockology, vol. XXI</ref><references group=\"klingon\"/>");
add("selser", "Bug 31374 regression check: nested strip items [4,0,0,0,0]", "1o4bkp1\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [2,4,0,0,0]", "1446r5o\n\n{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\nvqvr5f<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [2,2,0,3,0]", "13xsxuv\n\n{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\njqs94r\n<references group=Note /><references />");
add("selser", "Bug 31374 regression check: nested strip items [4,0,0,4,0]", "17opofg\n<references group=Note />1xq549u<references />");
add("selser", "Bug 31374 regression check: nested strip items [0,2,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\n616naz\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [1,2,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\nni4m5\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [2,0,0,0,0]", "1or99k3\n\n{{#tag:ref|note<ref>reference</ref>|group=Note}}\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [0,4,0,4,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\n1wykqkv<references group=Note />1q046uc<references />");
add("selser", "Bug 31374 regression check: nested strip items [2,2,0,0,0]", "1yn2ujl\n\n{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\nzhya21\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [3,4,0,3,0]", "13gz2oh<references group=Note /><references />");
add("selser", "Bug 31374 regression check: nested strip items [1,3,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [2,3,0,2,0]", "1d5hveq\n\n{{#tag:ref|note<ref>reference</ref>|group=Note}}<references group=Note />qqc5vy\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [1,0,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [0,4,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\n1t95tk<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [0,3,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [0,0,0,4,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n<references group=Note />sh9tjf<references />");
add("selser", "Bug 31374 regression check: nested strip items [3,0,0,0,0]", "\n<references group=Note />\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [0,0,0,2,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n<references group=Note />1l28h2z\n<references />");
add("selser", "Bug 31374 regression check: nested strip items [1,4,0,0,0]", "{{#tag:ref|note<ref>reference</ref>|group=Note}}\n\n5see0h<references group=Note />\n<references />");
add("selser", "Bug 13073 regression check: wrapped <references> [0,3,3]", "<ref>\nfoo\n</ref>\n");
add("selser", "Bug 13073 regression check: wrapped <references> [1,0,3]", "<ref>\nfoo\n</ref>\n");
add("selser", "Bug 13073 regression check: wrapped <references> [2,0,3]", "1eulfp2\n\n<ref>\nfoo\n</ref>\n");
add("selser", "Bug 13073 regression check: wrapped <references> [0,4,1]", "<ref>\nfoo\n</ref>\n\n1uo6dw<div data-foobar=\"nadb4v\"><references /></div>");
add("selser", "Bug 13073 regression check: wrapped <references> [0,0,1]", "<ref>\nfoo\n</ref>\n<div data-foobar=\"1xpxwf9\"><references /></div>");
add("selser", "Bug 13073 regression check: wrapped <references> [0,3,1]", "<ref>\nfoo\n</ref><div data-foobar=\"1vxoixa\"><references /></div>");
add("selser", "Bug 13073 regression check: wrapped <references> [0,4,0]", "<ref>\nfoo\n</ref>\n\nqffagq<div><references /></div>");
add("selser", "Bug 13073 regression check: wrapped <references> [1,2,0]", "<ref>\nfoo\n</ref>\n\na07t9n\n<div><references /></div>");
add("selser", "Bug 13073 regression check: wrapped <references> [0,0,4]", "<ref>\nfoo\n</ref>\n\n13gonk\n");
add("selser", "<ref> with no name and no content. [[3,0],4,0]", "<ref></ref>\n\nohfvpx");
add("selser", "<ref> with no name and no content. [0,2,0]", "Bla.<ref></ref>\n\npn6x71\n");
add("selser", "<ref> with no name and no content. [2,0,0]", "1otmssn\n\nBla.<ref></ref>");
add("selser", "<ref> with no name and no content. [1,3,0]", "Bla.<ref></ref>");
add("selser", "<ref> with no name and no content. [4,0,0]", "1wg5cj4\n");
add("selser", "<ref> with no name and no content. [2,3,0]", "13m8hdy\n\nBla.<ref></ref>");
add("selser", "<ref> with no name and no content. [3,2,0]", "1ety4vs\n");
add("selser", "<ref> with no name and no content. [1,4,0]", "Bla.<ref></ref>\n\n6v7sxu");
add("selser", "<ref> with no name and no content. [2,2,0]", "1tdm9xw\n\nBla.<ref></ref>\n\nxbklbx\n");
add("selser", "<ref> with no name and no content. [1,0,0]", "Bla.<ref></ref>\n");
add("selser", "<ref> with no name and no content. [[3,0],0,0]", "<ref></ref>\n");
add("selser", "<ref> with no name and no content. [0,3,0]", "Bla.<ref></ref>");
add("selser", "<ref> with no name and no content. [1,2,0]", "Bla.<ref></ref>\n\nazodfi\n");
add("selser", "<ref> with no name and no content. [[3,0],3,0]", "<ref></ref>");
add("selser", "<ref> with no name and no content. [[2,0],3,0]", "slk5uoBla.<ref></ref>");
add("selser", "<ref> with no name and no content. [0,4,0]", "Bla.<ref></ref>\n\nbnx45z");
add("selser", "<ref> with no name and no content. [3,0,0]", "\n");
add("selser", "<ref> with no name and no content. [2,4,0]", "1dg2w9n\n\nBla.<ref></ref>\n\np5ul07");
add("selser", "<ref> with no name and no content. [[4,0],0,0]", "imr7ut<ref></ref>\n");
add("selser", "<ref> with an empty-string name parameter and no content. [0,4,0]", "Bla.<ref name=\"\"></ref>\n\ncyppav");
add("selser", "<ref> with an empty-string name parameter and no content. [2,3,0]", "1wlyw6b\n\nBla.<ref name=\"\"></ref>");
add("selser", "<ref> with an empty-string name parameter and no content. [[3,0],3,0]", "<ref name=\"\"></ref>");
add("selser", "<ref> with an empty-string name parameter and no content. [[4,0],2,0]", "ke1ftq<ref name=\"\"></ref>\n\n1tohng1\n");
add("selser", "<ref> with an empty-string name parameter and no content. [4,0,0]", "1bv4hgd\n");
add("selser", "<ref> with an empty-string name parameter and no content. [3,0,0]", "\n");
add("selser", "<ref> with an empty-string name parameter and no content. [0,3,0]", "Bla.<ref name=\"\"></ref>");
add("selser", "<ref> with an empty-string name parameter and no content. [1,0,0]", "Bla.<ref name=\"\"></ref>\n");
add("selser", "<ref> with an empty-string name parameter and no content. [[4,0],4,0]", "1j8fnx<ref name=\"\"></ref>\n\ntivd28");
add("selser", "<ref> with an empty-string name parameter and no content. [2,0,0]", "1pjxq6c\n\nBla.<ref name=\"\"></ref>");
add("selser", "<ref> with an empty-string name parameter and no content. [[2,0],0,0]", "6bqi1jBla.<ref name=\"\"></ref>\n");
add("selser", "<ref> with an empty-string name parameter and no content. [3,2,0]", "1odk7yh\n");
add("selser", "<ref> with an empty-string name parameter and no content. [3,4,0]", "1vax2x1");
add("selser", "<ref> with an empty-string name parameter and no content. [[3,0],4,0]", "<ref name=\"\"></ref>\n\n1puamwb");
add("selser", "<ref> with an empty-string name parameter and no content. [3,3,0]", "");
add("selser", "<ref> with an empty-string name parameter and no content. [0,2,0]", "Bla.<ref name=\"\"></ref>\n\nir8lis\n");
add("selser", "<ref> with an empty-string name parameter and no content. [2,4,0]", "174k5cy\n\nBla.<ref name=\"\"></ref>\n\ny1bnct");
add("selser", "<ref> with an empty-string name parameter and no content. [4,3,0]", "1jij3pf");
add("selser", "<ref> with an empty-string name parameter and no content. [1,4,0]", "Bla.<ref name=\"\"></ref>\n\niknz8z");
add("selser", "<ref> with a non-empty name parameter and no content. [4,0,0]", "11rs4la\n");
add("selser", "<ref> with a non-empty name parameter and no content. [3,2,0]", "1uh0cto\n");
add("selser", "<ref> with a non-empty name parameter and no content. [[2,0],2,0]", "gav99uBla.<ref name=\"void\"></ref>\n\nr5vlst\n");
add("selser", "<ref> with a non-empty name parameter and no content. [0,4,0]", "Bla.<ref name=\"void\"></ref>\n\nr7lbys");
add("selser", "<ref> with a non-empty name parameter and no content. [[4,0],3,0]", "1l6z8wu<ref name=\"void\"></ref>");
add("selser", "<ref> with a non-empty name parameter and no content. [1,3,0]", "Bla.<ref name=\"void\"></ref>");
add("selser", "<ref> with a non-empty name parameter and no content. [2,3,0]", "110ffhv\n\nBla.<ref name=\"void\"></ref>");
add("selser", "<ref> with a non-empty name parameter and no content. [2,0,0]", "1qhovxt\n\nBla.<ref name=\"void\"></ref>");
add("selser", "<ref> with a non-empty name parameter and no content. [0,2,0]", "Bla.<ref name=\"void\"></ref>\n\neyqmdo\n");
add("selser", "<ref> with a non-empty name parameter and no content. [3,0,0]", "\n");
add("selser", "<ref> with a non-empty name parameter and no content. [1,4,0]", "Bla.<ref name=\"void\"></ref>\n\n3jybx1");
add("selser", "<ref> with a non-empty name parameter and no content. [[3,0],2,0]", "<ref name=\"void\"></ref>\n\nnj05ni\n");
add("selser", "<ref> with a non-empty name parameter and no content. [0,3,0]", "Bla.<ref name=\"void\"></ref>");
add("selser", "<ref> with a non-empty name parameter and no content. [2,4,0]", "8tf7uu\n\nBla.<ref name=\"void\"></ref>\n\n7sv7bv");
add("selser", "<ref> with a non-empty name parameter and no content. [[2,0],4,0]", "1aulowoBla.<ref name=\"void\"></ref>\n\n11th9k6");
add("selser", "<ref> with a non-empty name parameter and no content. [4,3,0]", "1ml1bcw");
add("selser", "<ref> with a non-empty name parameter and no content. [3,4,0]", "11q5mfj");
add("selser", "<ref> with a non-empty name parameter and no content. [[2,0],3,0]", "1fli58lBla.<ref name=\"void\"></ref>");
add("selser", "<ref> with a non-empty name parameter and no content. [[2,0],0,0]", "v0qv1iBla.<ref name=\"void\"></ref>\n");
add("selser", "<ref>s with the follow parameter [[3,0],0,0,4,3,0,0]", "<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\nxqyalt\n<references />");
add("selser", "<ref>s with the follow parameter [1,3,0,0,4,3,0]", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\nhqrwbi<references />");
add("selser", "<ref>s with the follow parameter [[3,0],0,0,2,[2],0,0]", "<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\nwfd1vh\n\n==new02f References ==\n<references />");
add("selser", "<ref>s with the follow parameter [0,0,1,3,0,0,0]", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n== References ==\n<references />");
add("selser", "<ref>s with the follow parameter [0,4,0,4,[2],3,0]", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\ndht9ly\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\n1ijup9y\n\n==zd10un References ==\n<references />");
add("selser", "<ref>s with the follow parameter [3,0,2,0,[3],2,0]", "\n1bjaryq\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\n==<nowiki/>==\n13mp93f\n<references />");
add("selser", "<ref>s with the follow parameter [0,4,[3,0],3,0,0,0]", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\nb1xm6f\n\n<ref>Second page footnote text.</ref>\n\n== References ==\n<references />");
add("selser", "<ref>s with the follow parameter [4,4,0,3,[3],0,0]", "1e4je6d\n\n1ngk4kr\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n==<nowiki/>==\n<references />");
add("selser", "<ref>s with the follow parameter [2,3,0,0,2,0,0]", "1w76sli\n\nPage one.<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\nlzh28u\n\n== References ==\n<references />");
add("selser", "<ref>s with the follow parameter [0,0,0,4,3,0,0]", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\nnc6mwu\n<references />");
add("selser", "<ref>s with the follow parameter [1,0,1,0,1,0,0]", "Page one.<ref name=\"beginning\">First page footnote text.</ref>\n\nPage two.<ref follow=\"beginning\">Second page footnote text.</ref>\n\n== References ==\n<references />");
add("selser", "<ref>s with the follow parameter [3,4,[4,0],0,[4],0,0]", "1pnw6s5\n\n18wphoo<ref follow=\"beginning\">Second page footnote text.</ref>\n\n==1egt7au==\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[2,0],2,0]", "7g93afPage one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n\nv9fx2h\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[4,0],3,0]", "jf0w9c<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref><references />");
add("selser", "<ref> with both name and follow parameters - invalid [1,0,0]", "Page one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[3,0],3,0]", "<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref><references />");
add("selser", "<ref> with both name and follow parameters - invalid [1,2,0]", "Page one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n\n9uvtao\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [1,4,0]", "Page one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n\n65h93m<references />");
add("selser", "<ref> with both name and follow parameters - invalid [2,4,0]", "1l0nj20\n\nPage one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n\n1lv442m<references />");
add("selser", "<ref> with both name and follow parameters - invalid [0,4,0]", "Page one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n\nqyc0hb<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[4,0],0,0]", "wm8by6<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[3,0],0,0]", "<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [2,0,0]", "13m01ms\n\nPage one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[2,0],4,0]", "2c055zPage one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n\ni4rk82<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[2,0],0,0]", "1saazdPage one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref>\n<references />");
add("selser", "<ref> with both name and follow parameters - invalid [[2,0],3,0]", "acc3sgPage one.<ref name=\"the-name\" follow=\"the-name\">This ref is invalid.</ref><references />");
add("selser", "Multiple definition (outside <references/>) [[0,4,0],4,0]", "<ref name=a>abc</ref>1lmk3w<ref name=a>def</ref>\n\n1eye0ty<references />");
add("selser", "Multiple definition (outside <references/>) [2,0,0]", "1c7qb6n\n\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n<references />");
add("selser", "Multiple definition (outside <references/>) [[0,2,0],0,0]", "<ref name=a>abc</ref>c8fyp1\n<ref name=a>def</ref>\n<references />");
add("selser", "Multiple definition (outside <references/>) [1,3,0]", "<ref name=a>abc</ref>\n<ref name=a>def</ref><references />");
add("selser", "Multiple definition (outside <references/>) [[0,3,0],4,0]", "<ref name=a>abc</ref><ref name=a>def</ref>\n\na6uxj4<references />");
add("selser", "Multiple definition (outside <references/>) [2,2,0]", "18a7rv4\n\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n\njlrdr9\n<references />");
add("selser", "Multiple definition (outside <references/>) [[0,2,0],2,0]", "<ref name=a>abc</ref>ju4hvk\n<ref name=a>def</ref>\n\notv6i7\n<references />");
add("selser", "Multiple definition (outside <references/>) [[0,3,0],3,0]", "<ref name=a>abc</ref><ref name=a>def</ref><references />");
add("selser", "Multiple definition (outside <references/>) [0,2,0]", "<ref name=a>abc</ref>\n<ref name=a>def</ref>\n\n8tji3b\n<references />");
add("selser", "Multiple definition (outside <references/>) [0,3,0]", "<ref name=a>abc</ref>\n<ref name=a>def</ref><references />");
add("selser", "Multiple definition (outside <references/>) [[0,2,0],3,0]", "<ref name=a>abc</ref>ih0ams\n<ref name=a>def</ref><references />");
add("selser", "Multiple definition (outside <references/>) [1,0,0]", "<ref name=a>abc</ref>\n<ref name=a>def</ref>\n<references />");
add("selser", "Multiple definition (outside <references/>) [[0,4,0],3,0]", "<ref name=a>abc</ref>1cgp5ih<ref name=a>def</ref><references />");
add("selser", "Multiple definition (outside <references/>) [[0,3,0],0,0]", "<ref name=a>abc</ref><ref name=a>def</ref>\n<references />");
add("selser", "Multiple definition (inside <references/>) [2,0,0]", "1shzt7i\n\n<ref name=a />\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [0,4,0]", "<ref name=a />\n\nj2d2v3<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [3,3,0]", "<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [4,0,0]", "1mq5xi9\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [3,2,0]", "1voaewx\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [3,0,0]", "\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [1,3,0]", "<ref name=a /><references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [2,2,0]", "17efii4\n\n<ref name=a />\n\ntm836o\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [0,2,0]", "<ref name=a />\n\n1wz7p7g\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [0,3,0]", "<ref name=a /><references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [1,2,0]", "<ref name=a />\n\nm81v1p\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [2,4,0]", "xf4te0\n\n<ref name=a />\n\n2z0p1t<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [4,3,0]", "1s3isxl<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [3,4,0]", "r0flg2<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [4,2,0]", "lh5k9l\n\nng6fcw\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [1,0,0]", "<ref name=a />\n<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [1,4,0]", "<ref name=a />\n\nx94ao8<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [2,3,0]", "c8cwx3\n\n<ref name=a /><references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside <references/>) [4,4,0]", "cmo8ee\n\ng752d7<references>\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [3,0,0]", "\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [0,3,0]", "<ref name=a>abc</ref><references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [1,0,0]", "<ref name=a>abc</ref>\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [0,2,0]", "<ref name=a>abc</ref>\n\nudes3p\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [1,3,0]", "<ref name=a>abc</ref><references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [2,2,0]", "1crcki5\n\n<ref name=a>abc</ref>\n\npju9zv\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [4,0,0]", "16j4odm\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [1,2,0]", "<ref name=a>abc</ref>\n\n1c4red\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [0,4,0]", "<ref name=a>abc</ref>\n\n2kee5s<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [4,4,0]", "1lshcq8\n\n1harbih<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [2,0,0]", "1ifluww\n\n<ref name=a>abc</ref>\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [2,4,0]", "j3dwcn\n\n<ref name=a>abc</ref>\n\n129zho0<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [4,2,0]", "1sz17ot\n\n1j56s11\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [4,3,0]", "rqhfci<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [2,3,0]", "1ab7bmd\n\n<ref name=a>abc</ref><references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [3,4,0]", "1ikkqqz<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [1,4,0]", "<ref name=a>abc</ref>\n\ns1jxel<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [3,3,0]", "<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (mixed outside/inside) [3,2,0]", "gpcp3u\n<references>\n<ref name=a>def</ref>\n</references>");
add("selser", "Multiple definition (inside {{#tag:references}}) [2,4,0]", "1aft3zn\n\n<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\nyyf7ac");
add("selser", "Multiple definition (inside {{#tag:references}}) [1,3,0]", "<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}");
add("selser", "Multiple definition (inside {{#tag:references}}) [1,2,0]", "<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\n6kbswz\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [0,2,0]", "<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\n5bho14\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [[0,3,0],2,0]", "<ref name=a />{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\nlt0dsi\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [4,2,0]", "1581laa\n\n1w4nfjk\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [0,4,0]", "<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\n3s1r49");
add("selser", "Multiple definition (inside {{#tag:references}}) [2,2,0]", "170te2t\n\n<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\noa4pku\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [3,4,0]", "1ict8e5");
add("selser", "Multiple definition (inside {{#tag:references}}) [[0,3,0],0,0]", "<ref name=a />{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [3,2,0]", "1r7p2dc\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [[0,2,0],4,0]", "<ref name=a />v9lo28\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\n1505q9s");
add("selser", "Multiple definition (inside {{#tag:references}}) [1,0,0]", "<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [[0,4,0],0,0]", "<ref name=a />xg7njs{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [4,3,0]", "10oobdf");
add("selser", "Multiple definition (inside {{#tag:references}}) [[0,2,0],0,0]", "<ref name=a />wvocua\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [4,0,0]", "c6o25q\n");
add("selser", "Multiple definition (inside {{#tag:references}}) [2,3,0]", "r5lb7d\n\n<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}");
add("selser", "Multiple definition (inside {{#tag:references}}) [1,4,0]", "<ref name=a />\n{{#tag:references|\n<ref name=a>abc</ref>\n<ref name=a>def</ref>\n}}\n\n1sdxf2c");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[0,0,2,0,3,0],0,0]", "A <ref name=\"x\">foo</ref>8o50yh\nB <ref name=\" x \" /><ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[0,0,4,0,2,0],0,0]", "A <ref name=\"x\">foo</ref>7oni2y<ref name=\" x \" />wcnop\nC <ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[0,0,3,0,0,0],3,0]", "A <ref name=\"x\">foo</ref><ref name=\" x \" />\nC <ref name= x  /><references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[4,0,3,0,3,0],0,0]", "ie0a2h<ref name=\"x\">foo</ref><ref name=\" x \" /><ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [2,0,0]", "1vzemgq\n\nA <ref name=\"x\">foo</ref>\nB <ref name=\" x \" />\nC <ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[3,0,4,0,0,0],4,0]", "<ref name=\"x\">foo</ref>56lkzv<ref name=\" x \" />\nC <ref name= x  />\n\nn8go92<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [1,2,0]", "A <ref name=\"x\">foo</ref>\nB <ref name=\" x \" />\nC <ref name= x  />\n\ndpqatn\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [0,4,0]", "A <ref name=\"x\">foo</ref>\nB <ref name=\" x \" />\nC <ref name= x  />\n\nkb5zhd<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[4,0,2,0,0,0],0,0]", "na5zcf<ref name=\"x\">foo</ref>1w7hqb4\nB <ref name=\" x \" />\nC <ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[3,0,3,0,0,0],0,0]", "<ref name=\"x\">foo</ref><ref name=\" x \" />\nC <ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[0,0,4,0,3,0],4,0]", "A <ref name=\"x\">foo</ref>xzdr22<ref name=\" x \" /><ref name= x  />\n\nfdyjh8<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[4,0,0,0,3,0],2,0]", "id09py<ref name=\"x\">foo</ref>\nB <ref name=\" x \" /><ref name= x  />\n\n14epo21\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[3,0,3,0,0,0],2,0]", "<ref name=\"x\">foo</ref><ref name=\" x \" />\nC <ref name= x  />\n\n2yz5rl\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [1,0,0]", "A <ref name=\"x\">foo</ref>\nB <ref name=\" x \" />\nC <ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[3,0,0,0,0,0],3,0]", "<ref name=\"x\">foo</ref>\nB <ref name=\" x \" />\nC <ref name= x  /><references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[3,0,4,0,2,0],0,0]", "<ref name=\"x\">foo</ref>1rpjo1b<ref name=\" x \" />rjd0zn\nC <ref name= x  />\n<references />");
add("selser", "Ref: 3. spaces in ref-names should be ignored [[0,0,0,0,4,0],0,0]", "A <ref name=\"x\">foo</ref>\nB <ref name=\" x \" />xidipo<ref name= x  />\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [0,2,0]", "A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\nrfr4d1\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [[2,0],0,0]", "c3i169A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [[2,0],4,0]", "8zihkxA <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\ns392cx<references />");
add("selser", "Ref: 5. body should accept generic wikitext [2,3,0]", "1gmyrnh\n\nA <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref><references />");
add("selser", "Ref: 5. body should accept generic wikitext [1,0,0]", "A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [1,2,0]", "A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\ncqu2ea\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [0,3,0]", "A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref><references />");
add("selser", "Ref: 5. body should accept generic wikitext [2,0,0]", "6ee4rt\n\nA <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [[4,0],0,0]", "e7jmj9<ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [[3,0],0,0]", "<ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n<references />");
add("selser", "Ref: 5. body should accept generic wikitext [0,4,0]", "A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n1ru20lv<references />");
add("selser", "Ref: 5. body should accept generic wikitext [[3,0],3,0]", "<ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref><references />");
add("selser", "Ref: 5. body should accept generic wikitext [1,4,0]", "A <ref>\n This is a '''[[bolded link]]''' and this is a {{echo|transclusion}}\n</ref>\n\n1cqoj1h<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[3,0],2,0]", "<ref>\n foo\n bar\n baz\n</ref>\n\no81nla\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[3,0],0,0]", "<ref>\n foo\n bar\n baz\n</ref>\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[4,0],0,0]", "ptuvjm<ref>\n foo\n bar\n baz\n</ref>\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[2,0],3,0]", "p8wx8eA <ref>\n foo\n bar\n baz\n</ref><references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [1,0,0]", "A <ref>\n foo\n bar\n baz\n</ref>\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [1,4,0]", "A <ref>\n foo\n bar\n baz\n</ref>\n\n4n3og5<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [2,2,0]", "1mkq2nw\n\nA <ref>\n foo\n bar\n baz\n</ref>\n\nlkdmcw\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [0,4,0]", "A <ref>\n foo\n bar\n baz\n</ref>\n\ne6tbf7<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[4,0],2,0]", "t1xo79<ref>\n foo\n bar\n baz\n</ref>\n\nohzvcd\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [0,3,0]", "A <ref>\n foo\n bar\n baz\n</ref><references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[2,0],2,0]", "65l2gkA <ref>\n foo\n bar\n baz\n</ref>\n\n9m5osh\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [2,3,0]", "1ps8i5m\n\nA <ref>\n foo\n bar\n baz\n</ref><references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [1,2,0]", "A <ref>\n foo\n bar\n baz\n</ref>\n\n51p9mr\n\n<references />");
add("selser", "Ref: 6. indent-pres should not be output in ref-body [[2,0],0,0]", "lelu0jA <ref>\n foo\n bar\n baz\n</ref>\n\n<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [2,4,0]", "11gmkak\n\nA <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\nsk6g75<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [1,0,0]", "A <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\n<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [2,0,0]", "19hkhl7\n\nA <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\n<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [[4,0],3,0]", "mhw4l<ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref><references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [0,4,0]", "A <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\n17wtnne<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [2,3,0]", "8w30ur\n\nA <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref><references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [1,4,0]", "A <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\nvrpcfk<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [0,2,0]", "A <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\ndbwcqg\n\n<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [[4,0],0,0]", "z8di7p<ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\n<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [[3,0],0,0]", "<ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\n<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [1,3,0]", "A <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref><references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [[2,0],4,0]", "66snq8A <ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\nwxakw7<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [[4,0],4,0]", "1m1b33i<ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref>\n\n4e9v8b<references />");
add("selser", "Ref: 7. No p-wrapping in ref-body [[3,0],3,0]", "<ref>\nfoo\n\nbar\n\n\nbaz\n\n\n\nbooz\n</ref><references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[3,0,0],0,0]", "<ref> foo {{echo|</ref> B C}}\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [0,2,0]", "A <ref> foo {{echo|</ref> B C}}\n\nndg5c4\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[0,0,2],2,0]", "A <ref> foo {{echo|</ref><nowiki>b5njrh B C}}</nowiki>\n\na1gxhc\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [1,4,0]", "A <ref> foo {{echo|</ref> B C}}\n\nu9j3ix<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [0,3,0]", "A <ref> foo {{echo|</ref> B C}}<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[2,0,0],0,0]", "1ulfrvA <ref> foo {{echo|</ref> B C}}\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[2,0,4],0,0]", "hmwoofA <ref> foo {{echo|</ref>14n13je\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [0,4,0]", "A <ref> foo {{echo|</ref> B C}}\n\n1pwtjex<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[4,0,3],4,0]", "f35roz<ref> foo {{echo|</ref>\n\nwy9pr3<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [1,0,0]", "A <ref> foo {{echo|</ref> B C}}\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[3,0,0],2,0]", "<ref> foo {{echo|</ref> B C}}\n\nukwq6i\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[4,0,2],3,0]", "fypf0r<ref> foo {{echo|</ref><nowiki>az008s B C}}</nowiki><references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [2,0,0]", "1b905vn\n\nA <ref> foo {{echo|</ref> B C}}\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[2,0,0],2,0]", "7uxsg5A <ref> foo {{echo|</ref> B C}}\n\n1v27tzn\n\n<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[0,0,4],4,0]", "A <ref> foo {{echo|</ref>1wsbfe6\n\nmic5ln<references />");
add("selser", "Ref: 8. transclusion wikitext has lower precedence [[4,0,2],4,0]", "ad938x<ref> foo {{echo|</ref><nowiki>9qwptv B C}}</nowiki>\n\n1j9wtig<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [2,3,0]", "179nhm1\n\nA <ref> foo <!--</ref> B C<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[4,0,0],4,0]", "sb6ksz<ref> foo <!--</ref> B C\n\ngzhiq3<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[0,0,4],3,0]", "A <ref> foo <!--</ref>aoy805<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[2,0,0],2,0]", "7hluh3A <ref> foo <!--</ref> B C\n\n11srvlw\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [1,0,0]", "A <ref> foo <!--</ref> B C\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[2,0,0],0,0]", "vfks5wA <ref> foo <!--</ref> B C\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[4,0,4],2,0]", "kix7sr<ref> foo <!--</ref>13rspfe\n\n1gyggbp\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[3,0,4],0,0]", "<ref> foo <!--</ref>a6c568\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [1,2,0]", "A <ref> foo <!--</ref> B C\n\n1ih18bl\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[2,0,0],4,0]", "1wmw3rhA <ref> foo <!--</ref> B C\n\n1nvcezs<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[4,0,0],3,0]", "1sddkuq<ref> foo <!--</ref> B C<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [2,0,0]", "17eaulh\n\nA <ref> foo <!--</ref> B C\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[0,0,2],0,0]", "A <ref> foo <!--</ref>a679pd B C\n<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[3,0,3],4,0]", "<ref> foo <!--</ref>\n\n1tnuwn7<references />");
add("selser", "Ref: 9. unclosed comments should not leak out of ref-body [[0,0,4],2,0]", "A <ref> foo <!--</ref>1s5ppq0\n\n1hwfma7\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [1,3,0]", "A <ref> <b> foo </ref> B C<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [1,2,0]", "A <ref> <b> foo </ref> B C\n\nhegopy\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [2,0,0]", "15f069w\n\nA <ref> <b> foo </ref> B C\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[3,0,2],0,0]", "<ref> <b> foo </ref>swmzx0 B C\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [2,3,0]", "17ymgx4\n\nA <ref> <b> foo </ref> B C<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[2,0,0],0,0]", "lz8h9bA <ref> <b> foo </ref> B C\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[4,0,2],0,0]", "a7uih1<ref> <b> foo </ref>ta8yz B C\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[0,0,2],0,0]", "A <ref> <b> foo </ref>5ncz3l B C\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[3,0,4],0,0]", "<ref> <b> foo </ref>f7l4yc\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[4,0,0],0,0]", "v9dbkg<ref> <b> foo </ref> B C\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [0,3,0]", "A <ref> <b> foo </ref> B C<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[2,0,0],2,0]", "6wri75A <ref> <b> foo </ref> B C\n\n9k6iq\n\n<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [0,4,0]", "A <ref> <b> foo </ref> B C\n\ntgolxy<references />");
add("selser", "Ref: 10. Unclosed HTML tags should not leak out of ref-body [[0,0,4],3,0]", "A <ref> <b> foo </ref>6oadkh<references />");
add("selser", "Ref: 12. ref-tags act as trailing newline migration barrier [4,[2],3,2,4,[4],3,0]", "17pa34f\n\n1g9n9uza\n\ntns9fo\n\nb<!--the newline at the end of this line stays inside the p tag--> <ref />\n<ref />\n\nztzbr0\n\n16udisr\n<references />");
add("selser", "References: 9. Generate missing references list at the end [0,4,0,3,0]", "A <ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>\n\nm523wx");
add("selser", "References: 9. Generate missing references list at the end [4,4,0,4,0]", "1764b27\n\n1ssyey51i8g4aj");
add("selser", "References: 9. Generate missing references list at the end [[2,0,2,0],0,0,3,0]", "fnars4A <ref>foo</ref>a75i8b\nB <ref group=\"inexistent\">bar</ref>\n");
add("selser", "References: 9. Generate missing references list at the end [3,0,0,2,0]", "\n1mbxn6j\n");
add("selser", "References: 9. Generate missing references list at the end [[0,0,2,0],2,0,0,0]", "A <ref>foo</ref>qqtrep\nB <ref group=\"inexistent\">bar</ref>\n\n1tyoqe5\n");
add("selser", "References: 9. Generate missing references list at the end [4,2,0,3,0]", "1fe5ffa\n\n1ruvjir\n");
add("selser", "References: 9. Generate missing references list at the end [0,3,0,0,0]", "A <ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>");
add("selser", "References: 9. Generate missing references list at the end [[0,0,4,0],0,0,4,0]", "A <ref>foo</ref>wz4guj<ref group=\"inexistent\">bar</ref>\npbcyz2");
add("selser", "References: 9. Generate missing references list at the end [[0,0,4,0],0,0,3,0]", "A <ref>foo</ref>nl5b0<ref group=\"inexistent\">bar</ref>\n");
add("selser", "References: 9. Generate missing references list at the end [[3,0,0,0],3,0,0,0]", "<ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>");
add("selser", "References: 9. Generate missing references list at the end [3,3,0,0,0]", "");
add("selser", "References: 9. Generate missing references list at the end [[4,0,4,0],2,0,4,0]", "z1d44k<ref>foo</ref>12zna7d<ref group=\"inexistent\">bar</ref>\n\n1rtnhuh\n1w1j39l");
add("selser", "References: 9. Generate missing references list at the end [4,3,0,2,0]", "1mbcfde1utf8bj\n");
add("selser", "References: 9. Generate missing references list at the end [2,3,0,0,0]", "1dmiax9\n\nA <ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>");
add("selser", "References: 9. Generate missing references list at the end [[4,0,0,0],3,0,0,0]", "r7gaia<ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>");
add("selser", "References: 9. Generate missing references list at the end [[4,0,0,0],0,0,2,0]", "1kbqga8<ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>\n1y952id\n");
add("selser", "References: 9. Generate missing references list at the end [1,3,0,0,0]", "A <ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>");
add("selser", "References: 9. Generate missing references list at the end [0,0,0,4,0]", "A <ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>xixkmh");
add("selser", "References: 9. Generate missing references list at the end [[4,0,0,0],3,0,3,0]", "jqf638<ref>foo</ref>\nB <ref group=\"inexistent\">bar</ref>");
add("selser", "References: 9. Generate missing references list at the end [3,0,0,0,0]", "\n");

// ### DO NOT REMOVE THIS LINE ### (end of automatically-generated section)


if (typeof module === 'object') {
	module.exports.testBlackList = testBlackList;
}
